import pickle
import base64
from cryptography.fernet import Fernet
from cryptography.hazmat.primitives import hashes
from cryptography.hazmat.primitives.kdf.pbkdf2 import PBKDF2HMAC
import os


def generate_key(password, salt):
    kdf = PBKDF2HMAC(
        algorithm=hashes.SHA256(),
        length=32,
        salt=salt,
        iterations=100000,
    )
    return base64.urlsafe_b64encode(kdf.derive(password))

module_path = os.path.abspath(os.path.dirname(__file__))
enc_file_path = os.path.join(module_path, 'financial_constants.enc')

with open(enc_file_path, 'rb') as file:
    salt = file.read(16) 
    encrypted_data = file.read()

password = input("input your password").encode()

key = generate_key(password, salt)

# create encryption object
cipher_suite = Fernet(key)

# decryption
decrypted_data = cipher_suite.decrypt(encrypted_data)

# Load data from Pickle
loaded_data = pickle.loads(decrypted_data)

korea_financial_column = loaded_data["korea_financial_column"]
fmp_FMP_field = loaded_data["fmp_FMP_field"]
fmp_core_cols = loaded_data["fmp_core_cols"]
fmp_is_cols = loaded_data["fmp_is_cols"]
fmp_bs_cols = loaded_data["fmp_bs_cols"]
fmp_cf_cols = loaded_data["fmp_cf_cols"]




import pandas as pd
import numpy as np
import time
from datetime import datetime
import chromedriver_autoinstaller
import re
from bs4 import BeautifulSoup
from selenium import webdriver
from tqdm import tqdm
from selenium.webdriver.common.by import By
from selenium.webdriver.chrome.service import Service
from selenium.common.exceptions import NoSuchElementException
from selenium.webdriver import ActionChains
from webdriver_manager.chrome import ChromeDriverManager


class information:
    def __init__(self):
        self.print_information()

    def print_information(self):
        print("""
        The library is divided into two parts. \n
        Investing_Crawler, a library that collects data, Investing_Cleanse, a library that processes data \n
        -------------------------------------------------- ----------------------------------------\n
        The function of Investing_Crawler is shown below. \n
        DriverSettings() is a Selenium Chrome driver settings function. \n
        download_historial() is a function that collects past stock price data. \n
        collect() is a function that collects data from investing.com. \n
        -------------------------------------------------- ----------------------------------------\n
        Investing_Cleanse will proceed as soon as you run the class. \n
        """)

    def country(self):
        print("""
        japan, hong-kong, malaysia, south-korea
        singapore, thailand, vietnam, indonesia
        india ,united-states, spain, switzerland
        australia, united-kingdom, france, italy
        germany, netherlands, mexico, colombia, canada
        """)

class Investing_Crawler:

    def __init__(self, path):
        """
        Please insert the investing column Excel in the path.
        """
        self.base_url = 'https://au.investing.com'
        self.PROFILE_suffix = '-company-profile'
        self.IS_suffix = '-income-statement'
        self.BS_suffix = '-balance-sheet'
        self.CF_suffix = '-cash-flow'
        self.column_listup = pd.read_excel(path)
        # Investing.com account list. Accounting subjects for general corporations, banks, and insurance businesses were different.
        self.all_bs_cols = pd.concat([self.column_listup['BS'],self.column_listup['BANK BS'], self.column_listup['INSURANCE BS']]).dropna()
        self.all_is_cols = pd.concat([self.column_listup['IS'],self.column_listup['BANK IS'], self.column_listup['INSURANCE IS']]).dropna()
        self.all_cf_cols = pd.concat([self.column_listup['CF'],self.column_listup['BANK CF'], self.column_listup['INSURANCE CF']]).dropna()
        self.all_cols = pd.concat([self.column_listup['BS'],self.column_listup['BANK BS'], self.column_listup['INSURANCE BS'],self.column_listup['IS'],self.column_listup['BANK IS'], self.column_listup['INSURANCE IS'],
                            self.column_listup['CF'],self.column_listup['BANK CF'],self.column_listup['INSURANCE CF']]).dropna()

        # Remove the overlapping parts among the account subjects of general corporations, banks, and insurance businesses to use the union without duplicates. 
        # drop_duplicates removes only one of the duplicates
        self.all_cols = self.all_cols.drop_duplicates()
        self.company_links = []

    def DriverSettings(self, Turn_off_warning = False, linux_mode = False) -> None:
        """
        This function sets the driver.
        If linux mode is set to True, collection is possible in the background.
        However, actions such as clicks cannot be taken.
        """
        if Turn_off_warning == True: self.TurnOffWarning()
        chrome_ver = chromedriver_autoinstaller.get_chrome_version().split('.')[0]  # Check chromedriver version
        chrome_options = webdriver.ChromeOptions()
        chrome_options.add_argument("--incognito") # incognito mode
        if linux_mode == True: chrome_options.add_argument("--headless") # Display mode without Linux GUI
        chrome_options.add_argument("--no-sandbox") # Prevent access to resources
        chrome_options.add_argument("--disable-setuid-sandbox") # Prevent chrome crashes
        chrome_options.add_argument("--disable-dev-shm-usage") # Prevent out of memory errors
        chrome_options.add_experimental_option('excludeSwitches', ['enable-logging'])
        try: # Chrome Driver
            self.driver = webdriver.Chrome(service=Service(ChromeDriverManager().install()), options=chrome_options)   
        except:
            chromedriver_autoinstaller.install(True)
            self.driver = webdriver.Chrome(service=Service(ChromeDriverManager().install()), options=chrome_options)
        # Prevent WebDruverException Error Designate as an existing driver version

    def download_historial(self, all_atag_maintable, url):
        company_links = []
        for a in all_atag_maintable:
            company_link = a.attrs["href"]
            company_links.append(company_link)
        company_table_css = self.driver.find_element(By.CSS_SELECTOR, 'div[data-test="dynamic-table"]')
        company_table_html = company_table_css.get_attribute('outerHTML')
        company_table = pd.read_html(company_table_html)[0]
        stock_history_company_links = [i+"-historical-data" for i in company_links]
        for idx, name in tqdm(zip(stock_history_company_links[912:], company_table["Name"][912:])):
            self.driver.get("https://au.investing.com/" + idx)
            time.sleep(2)
            try: click_time = self.driver.find_element(By.XPATH, '//*[@id="history-timeframe-selector"]').click()
            except NoSuchElementException:
                self.driver.get("https://au.investing.com/" + idx)
                click_time = self.driver.find_element(By.XPATH, '//*[@id="history-timeframe-selector"]').click()
            click_month = self.driver.find_element(By.XPATH, '//*[@id="react-select-2-option-1"]').click()
            try: historical_data = self.driver.find_element(By.XPATH, '//*[@id="__next"]/div[2]/div/div/div[2]/main/div/div[6]/div/div/div[2]/div[2]')
            except NoSuchElementException: 
                try:
                    historical_data = self.driver.find_element(By.XPATH, '//*[@id="__next"]/div[2]/div/div/div[2]/main/div/div[7]/div/div/div[2]/div[2]')
                except: 
                    self.driver.refresh()
                    click_time = self.driver.find_element(By.XPATH, '//*[@id="history-timeframe-selector"]').click()
                    click_month = self.driver.find_element(By.XPATH, '//*[@id="react-select-2-option-1"]').click()
                    try: historical_data = self.driver.find_element(By.XPATH, '//*[@id="__next"]/div[2]/div/div/div[2]/main/div/div[6]/div/div/div[2]/div[2]')
                    except NoSuchElementException: historical_data = self.driver.find_element(By.XPATH, '//*[@id="__next"]/div[2]/div/div/div[2]/main/div/div[7]/div/div/div[2]/div[2]')
            time.sleep(2)
            historical_data.click()
            input_element = historical_data.find_element(By.TAG_NAME, 'input')
            while input_element.get_attribute('value') != "2018-01-01":
                input_element.clear()
                input_element.send_keys("20180101")
            month_button = historical_data.find_element(By.TAG_NAME, 'button')
            month_button.send_keys("\n") 
            time.sleep(3)
            a = historical_data.find_element(By.TAG_NAME, 'a')
            a.click()
        self.driver.get(url)

    def collect(self, country, official_countryName ,save_dir, isSingapore=False, download_history=False) : 
        first_url = f'https://au.investing.com/equities/{country.lower()}'
        self.driver.get(first_url)
        time.sleep(8)
        breakpoint()
        # Replace the stocks displayed on the page with 'all stocks'
        try : 
            if isSingapore == False : 
                select_box = self.driver.find_element(By.XPATH,'//*[@id="index-select"]/div[1]').click()
                select_all_stock = self.driver.find_element(By.XPATH,'//*[@id="index-select"]/div[2]/div/div/div[1]').click()
            else : 
                pass
            time.sleep(5)

        # Avoid errors. If an error occurs due to an advertisement pop-up window, use except to avoid the error
        except : 
            # Close ad pop-up window 
            self.driver.find_element(By.XPATH, '//*[@id="PromoteSignUpPopUp"]/div[2]/i').click()

            # Singapore does not have a tab to select all stock. So, the part where you click the all stocks button
            # skip over
            if isSingapore == False : 
                select_box = self.driver.find_element(By.XPATH,'//*[@id="stocksFilter"]').click()
                select_all_stock = self.driver.find_element(By.XPATH,'//*[@id="all"]').click()
            else : 
                pass
            time.sleep(5)

        # Investing.com is a dynamic page.
        # Scroll all the way down once to use BeautifulSoup.
        SCROLL_PAUSE_SEC = 3

        last_height = self.driver.execute_script("return document.body.scrollHeight")
        while True:
            
            try:
                some_tag = self.driver.find_element(By.XPATH, '//*[@id="__next"]/div[2]/div/div/div[2]/main/div[3]/div[2]/div')
                action = ActionChains(self.driver)
                action.move_to_element(some_tag).perform()
                some_tag.click()
            except NoSuchElementException: break
            # scroll down to the end
            self.driver.execute_script("window.scrollTo(0, document.body.scrollHeight);")

            # Wait for SCROLL_PAUSE_SEC
            time.sleep(SCROLL_PAUSE_SEC)
            
            # Get scroll height back after scrolling down
            new_height = self.driver.execute_script("return document.body.scrollHeight")
            if new_height == last_height:
                break
            last_height = new_height

        time.sleep(20)
        soup = BeautifulSoup(self.driver.page_source,"html.parser")

        # Get the stock table.
        maintable = soup.find('div', {'data-test': 'dynamic-table'})

        # From main stock table to company page
        all_atag_maintable = maintable.find_all('a')

        # If you use soup's find method to get it, not only the link but also other information will be brought back.
        # Therefore, only pure href links are extracted with the for statement below and stored in company_links.
        for a in all_atag_maintable:
            company_link = a.attrs["href"]
            self.company_links.append(company_link)

        # Version 3
        if download_history == True: self.download_historial(all_atag_maintable, first_url)
        else: pass
        wait_time = 3 


        # Remove the overlapping parts among the account subjects of general corporations, banks, and insurance businesses to use the union without duplicates.
        # drop_duplicates removes only one of the duplicates, then lists
        all_cols = list(self.all_cols.drop_duplicates())
        all_cols = all_cols+['is_unit','bs_unit','cf_unit','is_time','bs_time','cf_time','report_type','company_name','industry_info','sector_info',
                            'address_info','phone_info','fax_info','webpage_info','source','gathering_time','PIC']

        # Add company name and general information to all_cols.

        self.result_df = pd.DataFrame()
        crawling_failed_companies = []
        for company in self.company_links : 

            profile_url = self.base_url+company+self.PROFILE_suffix
            bs_url = self.base_url+company+self.BS_suffix
            is_url = self.base_url+company+self.IS_suffix
            cf_url = self.base_url+company+self.CF_suffix

            try : 
                # If you look at company_link, there are companies with unusual links that have 'cid=' attached to the end of the address. These companies need to be dealt with.
                if company.__contains__('cid=') : 
                    index = company.find('?')
                    profile_url =  self.base_url+company[:index]+self.PROFILE_suffix+company[index:]
                    bs_url = self.base_url+company[:index] +self.BS_suffix+company[index:]
                    is_url = self.base_url+company[:index] +self.IS_suffix+company[index:]
                    cf_url = self.base_url+company[:index] +self.CF_suffix+company[index:]

                ############## Company General Information #############
                # company profile page
                self.driver.get(profile_url)
                time.sleep(1)

                # description
                soup = BeautifulSoup(self.driver.page_source, "html.parser")
                desciption_info = soup.find('div', attrs = {'class' : 'companyProfileBody'}).text
                desciption_info = desciption_info.replace('\n','')

                profile_header = soup.find('div', attrs = {'class' : 'companyProfileHeader'}).text

                # Industry, Sector, Equity Type        
                industry_info =  profile_header.split('\n')[1].replace('Industry','')
                sector_info = profile_header.split('\n')[2].replace('Sector','')

                # contact info 
                address_info = soup.find('div', attrs = {'class' : 'companyAddress'}).text
                phone_info = soup.find('div', attrs = {'class' : 'companyPhone'}).text
                fax_info = soup.find('div', attrs = {'class' : 'companyFax'}).text
                webpage_info = soup.find('div', attrs = {'class' : 'companyWeb'}).text

                # remove special characters
                phone_info = phone_info.replace('\n','')
                phone_info = phone_info.replace('Phone','')
                fax_info = fax_info.replace('\n','')
                fax_info = fax_info.replace('Fax','')
                webpage_info = webpage_info.replace('\n','')
                webpage_info = webpage_info.replace('Web','')

                ############## Corporate financial information #########
                # Quarterly Income Statement
                self.driver.get(is_url)
                time.sleep(wait_time)

                for order, table in enumerate(pd.read_html(self.driver.page_source)) : 
                    if len(table) > 20 :
                        table_num = order
                        break

                df_income_Q = pd.read_html(self.driver.page_source)[table_num].dropna()
                df_is_unit = self.driver.find_element(By.XPATH,'//*[@id="leftColumn"]/div[10]').text            # Currency unit (Unit)
                company_name = self.driver.find_element(By.XPATH,'//*[@id="leftColumn"]/div[1]/h1').text             # extract company name

                # income statement dataframe preprocess
                df_income_Q = df_income_Q.T
                df_income_Q = df_income_Q.rename(columns = df_income_Q.iloc[0])        # set account subject as column name
                df_income_Q = df_income_Q.iloc[1:, :]                                # Delete row 0 and leave only the rest.

                df_income_Q['is_unit'] = df_is_unit                 # insert income statement currency into dataframe
                df_income_Q['is_time']= df_income_Q.index                  # Add 'time' to the dataframe
                df_income_Q['report_type'] = 'Quarter'                         # Mark the data frame as a quarterly report
                df_income_Q['company_name'] = company_name                     # insert company name into dataframe
                df_income_Q=df_income_Q.reset_index(drop=True) 

                # yearly Income Statement. 
                try : 
                    self.driver.find_element(By.XPATH,'//*[@id="leftColumn"]/div[8]/div[1]/a[1]').click()
                    time.sleep(2)
                except : 
                    self.driver.find_element(By.XPATH,'//*[@id="leftColumn"]/div[9]/div[1]/a[1]').click()
                    time.sleep(2)    

                for order, table in enumerate(pd.read_html(self.driver.page_source)) : 
                    if len(table) > 20 :
                        table_num = order
                        break

                df_income_A = pd.read_html(self.driver.page_source)[table_num].dropna()

                # income statement dataframe preprocess
                df_income_A = df_income_A.T
                df_income_A = df_income_A.rename(columns = df_income_A.iloc[0])        # set account subject as column name
                df_income_A = df_income_A.iloc[1:, :]                                # Delete row 0 and leave only the rest.

                df_income_A['is_unit'] = df_is_unit                 # insert income statement currency into dataframe
                df_income_A['is_time']= df_income_A.index                  # Add 'time' to the dataframe
                df_income_A['report_type'] = 'Annual'                          # Mark the data frame as a quarterly report
                df_income_A['company_name'] = company_name                     # insert company name into data frame
                df_income_A=df_income_A.reset_index(drop=True) 


                # balance data
                self.driver.get(bs_url)
                time.sleep(wait_time)

                for order, table in enumerate(pd.read_html(self.driver.page_source)) : 
                    if len(table) > 20 :
                        table_num = order


                # quarterly balance sheet
                df_balance_Q = pd.read_html(self.driver.page_source)[table_num].dropna()
                df_bs_unit = self.driver.find_element(By.XPATH,'//*[@id="leftColumn"]/div[10]').text           # Extract currency unit (Unit)
                time.sleep(wait_time)
                # balance sheet dataframe preprocess
                df_balance_Q = df_balance_Q.T
                df_balance_Q = df_balance_Q.rename(columns = df_balance_Q.iloc[0])    # set account subject as column name
                df_balance_Q = df_balance_Q.iloc[1:, :]                              # Delete row 0 and leave only the rest.
                ##df_balance = df_balance[bs_targets]
                df_balance_Q['bs_unit'] = df_bs_unit                  # insert cash flow currency unit into dataframe
                df_balance_Q['bs_time']= df_balance_Q.index
                #df_balance_Q['report_type'] = 'Quarter'                         # Mark the data frame as a quarterly report
                #df_balance_Q['company_name'] = company_name                     # insert company name into data frame
                df_balance_Q=df_balance_Q.reset_index(drop=True)


                # yearly balance sheet 
                try : 
                    self.driver.find_element(By.XPATH,'//*[@id="leftColumn"]/div[8]/div[1]/a[1]').click() # Click the Annual button
                except :
                    self.driver.find_element(By.XPATH,'//*[@id="leftColumn"]/div[9]/div[1]/a[1]').click() # Click the Annual button
                time.sleep(wait_time)

                for order, table in enumerate(pd.read_html(self.driver.page_source)) : 
                    if len(table) > 20 :
                        table_num = order
                        break

                df_balance_A = pd.read_html(self.driver.page_source)[table_num].dropna()
                df_bs_unit = self.driver.find_element(By.XPATH,'//*[@id="leftColumn"]/div[10]').text           # Extract currency unit (Unit)
                time.sleep(wait_time)
                # balance sheet dataframe preprocess
                df_balance_A = df_balance_A.T
                df_balance_A = df_balance_A.rename(columns = df_balance_A.iloc[0])     # set account subject as column name
                df_balance_A = df_balance_A.iloc[1:, :]                              # Delete row 0 and leave only the rest.
                ##df_balance = df_balance[bs_targets]
                df_balance_A['bs_unit'] = df_bs_unit                  # insert cash flow currency unit into dataframe
                df_balance_A['bs_time']= df_balance_A.index
                #df_balance_A['report_type'] = 'Annual'                          # Mark the data frame as an annual report
                #df_balance_A['company_name'] = company_name                     # insert company name into data frame
                df_balance_A = df_balance_A.reset_index(drop=True)

                # cash flow
                self.driver.get(cf_url)
                time.sleep(2)

                for order, table in enumerate(pd.read_html(self.driver.page_source)) : 
                    if len(table) > 20 :
                        table_num = order
                        break    

                # quarterly cashflow 
                df_cash_flow_Q = pd.read_html(self.driver.page_source)[table_num].dropna()
                df_cf_unit = self.driver.find_element(By.XPATH,'//*[@id="leftColumn"]/div[10]').text           # Extract currency unit (Unit)
                time.sleep(wait_time)

                df_cash_flow_Q = df_cash_flow_Q.T 
                df_cash_flow_Q = df_cash_flow_Q.rename(columns = df_cash_flow_Q.iloc[0])# set account subject as column name
                df_cash_flow_Q = df_cash_flow_Q.iloc[1:, :]                             # Delete row 0 and leave only the rest.
                df_cash_flow_Q['cf_unit'] = df_cf_unit                     # insert cash flow currency unit into dataframe
                df_cash_flow_Q['cf_time']= df_cash_flow_Q.index.get_level_values(0)  # cf는 period_ending, period_length Using two multi-indexes  
                df_cash_flow_Q = df_cash_flow_Q.reset_index(drop=True)

                # yearly cashflow 
                try : 
                    self.driver.find_element(By.XPATH,'//*[@id="leftColumn"]/div[8]/div[1]/a[1]').click() # Click the Annual button
                except :                         
                    self.driver.find_element(By.XPATH,'//*[@id="leftColumn"]/div[9]/div[1]/a[1]').click() # Click the Annual button
                time.sleep(wait_time)

                for order, table in enumerate(pd.read_html(self.driver.page_source)) : 
                    if len(table) > 20 :
                        table_num = order
                        break        

                df_cash_flow_A = pd.read_html(self.driver.page_source)[table_num].dropna()
                df_cash_flow_unit = self.driver.find_element(By.XPATH,'//*[@id="leftColumn"]/div[10]').text            # Extract currency unit (Unit)

                #cash flow dataframe preprocess
                df_cash_flow_A = df_cash_flow_A.T 
                df_cash_flow_A = df_cash_flow_A.rename(columns = df_cash_flow_A.iloc[0])# set account subject as column name
                df_cash_flow_A = df_cash_flow_A.iloc[1:, :]                             # Delete row 0 and leave only the rest.
                df_cash_flow_A['cf_unit'] = df_cf_unit                     # insert cash flow currency unit into dataframe
                df_cash_flow_A['cf_time']= df_cash_flow_A.index.get_level_values(0)  # cf는 period_ending, period_length Using two multi-indexes
                df_cash_flow_A = df_cash_flow_A.reset_index(drop=True) 

                # Integrate column by batch.
                # company_df_A is a business report obtained by pressing the 'Annual' button
                company_df_A = pd.concat([df_balance_A, df_income_A, df_cash_flow_A],axis=1) # Indicate that it is a business report in the data frame
                company_df_A['industry_info'] = industry_info
                company_df_A['sector_info'] = sector_info
                company_df_A['address_info'] = address_info
                company_df_A['phone_info'] = phone_info
                company_df_A['fax_info'] = fax_info
                company_df_A['webpage_info'] = webpage_info
                company_df_A['source'] = 'https://au.investing.com'+company+self.IS_suffix
                company_df_A['PIC'] = 'Nicholas'
                company_df_A['gathering_time'] = datetime.date.today()
                
                # company_df_Q is the quarterly report obtained by pressing the 'Quarter' button
                company_df_Q = pd.concat([df_balance_Q, df_income_Q, df_cash_flow_Q],axis=1)# Mark the data frame as a quarterly report
                company_df_Q['industry_info'] = industry_info
                company_df_Q['sector_info'] = sector_info
                company_df_Q['address_info'] = address_info
                company_df_Q['phone_info'] = phone_info
                company_df_Q['fax_info'] = fax_info
                company_df_Q['webpage_info'] = webpage_info
                company_df_Q['source'] = 'https://au.investing.com'+company+self.IS_suffix
                company_df_Q['PIC'] = 'Nicholas'
                company_df_Q['gathering_time'] = datetime.date.today()

                
                blank = pd.DataFrame(columns = all_cols)
                for code in blank.columns : 
                # Extract necessary information with the ifrs code of the correspondence table.
                    try : 
                        blank[code] = company_df_A[code] 
                    except : 
                        pass  

                self.result_df = self.result_df.append(blank)
                blank = pd.DataFrame(columns = all_cols)

                for code in blank.columns : 
                # Extract necessary information with the ifrs code of the correspondence table.
                    try : 
                        blank[code] = company_df_Q[code] 
                    except : 
                        pass  


                self.result_df = self.result_df.append(blank)

            except :
                # If an error occurs due to lack of financial information, add general information as well.
                ############## Company General Information #########
                #company profile page
                self.driver.get(profile_url)
                time.sleep(1)

                #  description
                soup = BeautifulSoup(self.driver.page_source, "html.parser")
                try : 
                    company_name = soup.find('h2').text.replace('Company Profile',"")
                    #  description
                    desciption_info = soup.find('div', attrs = {'class' : 'companyProfileBody'}).text
                    desciption_info = desciption_info.replace('\n','')
                except : 
                    print(f"A company without a profile {profile_url}")
                    print(f"A company without a profile {company}")
                    continue

                profile_header = soup.find('div', attrs = {'class' : 'companyProfileHeader'}).text

                # Industry, Sector, Equity Type        
                industry_info =  profile_header.split('\n')[1].replace('Industry','')
                sector_info = profile_header.split('\n')[2].replace('Sector','')

                # contact info 
                address_info = soup.find('div', attrs = {'class' : 'companyAddress'}).text
                phone_info = soup.find('div', attrs = {'class' : 'companyPhone'}).text
                fax_info = soup.find('div', attrs = {'class' : 'companyFax'}).text
                webpage_info = soup.find('div', attrs = {'class' : 'companyWeb'}).text


                phone_info = phone_info.replace('\n','')
                phone_info = phone_info.replace('Phone','')
                fax_info = fax_info.replace('\n','')
                fax_info = fax_info.replace('Fax','')
                webpage_info = webpage_info.replace('\n','')
                webpage_info = webpage_info.replace('Web','')


                blank = pd.DataFrame(columns = all_cols)
                blank['company_name'] = company_name
                blank['industry_info'] = industry_info
                blank['sector_info'] = sector_info
                blank['address_info'] = address_info
                blank['phone_info'] = phone_info
                blank['fax_info'] = fax_info
                blank['webpage_info'] = webpage_info
                blank['source'] = 'https://au.investing.com'+company+self.PROFILE_suffix
                blank['PIC'] = 'Nicholas'
                blank['gathering_time'] = datetime.date.today()


                for code in blank.columns : 
                    # Extract necessary information with the ifrs code of the correspondence table.
                    try : 
                        blank[code] = company_df_A[code] 
                    except : 
                        pass  
                
                self.result_df = self.result_df.append(blank)
                
        self.result_df['Country'] = official_countryName
        self.result_df.to_excel(save_dir ,index = False)
        self.driver.quit()
        
        return self.result_df 

class Investing_Cleanse: 
        
        
    def __init__(self, originalfile_dir, mapping_sheet_dir, Replace_missing_currency_code = True):
        self.data = pd.read_excel(originalfile_dir)
        # Remove rows without company name
        self.data = self.data.dropna(subset = ['company_name'])
        # Add a column called 'currency' corresponding to the currency code to data
        self.data['currency'] = self.data['bs_unit'].apply(lambda x : self.currency_extract(x))
        # Add column 'unit' to self.data
        self.data['unit'] = self.data['bs_unit'].apply(lambda x : self.unit_change(x))
        # Columns containing numeric data in the original data are from the 0th column to the 141st column.
        # Before changing the unit of numeric data with self.data['unit'], sometimes null values are entered as '-', so remove them first.
        for name in self.data.columns[:142] : self.data[name]= self.data[name].replace('-',np.nan)
        # Change units for numeric data.
        # Columns 0 to 113 contain numeric data
        for name in self.data.columns[:142] : self.data[name]= self.data[name].astype('float')*self.data['unit']
        # Add a column called 'stock_code' by extracting the ticker from the company name.
        self.data['stock_code'] = self.data['company_name'].apply(lambda x : self.ticker_extract(x))
        # Since we extracted the ticker from the company name, remove the ticker.
        self.data['company_name'] = self.data['company_name'].apply(lambda x : self.ticker_delete(x))
        # Process bs_time into date_time format to generate fiscal year and fiscal quarter
        # Add a column called 'ending' period.
        self.data['ending_period'] = self.data['bs_time'].apply(lambda x : self.ending_period_extract(x))
        # remove missing values
        # Remove missing values without ending_perod before extracting the fiscal year and fiscal quarter.
        self.data = self.data.dropna(subset=['ending_period'])
        # Fill Hebron star country code column
        try: self.data['hb_nation_code'] = self.data['Country'].apply(lambda x : self.hb_nation_code(x))
        except KeyError:
            country_name = originalfile_dir.split(".xlsx")[-1]
            self.data['hb_nation_code'] = self.hb_nation_code(country_name)
        # Replace or remove data without a currency code with a frequent value.
        if Replace_missing_currency_code == True : 
            try: self.data['currency'] = self.data['currency'].fillna(value = self.data['currency'].mode()[0])
            except: print("currency is miss")
            # Fill in the fields without a currency code with the most frequently used currency code for the country.
        else : self.data = self.data.dropna(subset=['currency'])
            # If currency code missing substitution is False, rows with missing values are removed.
        # Create a Fiscal year column corresponding to the fiscal year using ending_period
        self.data['Fiscal year'] = self.data['ending_period'].apply(lambda x : x.year)
        # Creating an ending_period column corresponding to the settlement date using ending_period

        self.data['ending_period'] = self.data['ending_period'].apply(lambda x : x.strftime("%Y%m%d"))
        # Processing to extract illiquid assets
        self.data['Total Assets - Total Current Assets'] = self.data['Total Assets']-self.data['Total Current Assets']
        # Process for extracting total liabilities and assets
        self.data['Total Liabilities + Total Equity'] = self.data['Total Liabilities']+self.data['Total Equity']
        # report type
        self.data['report_type'] = self.data['report_type'].apply(lambda x : self.report_type_generator(x))
        self.mapping_sheet = pd.read_excel(mapping_sheet_dir)
        self.mapping_dic = self.mapping_sheet.set_index('2022 field name').T.to_dict('index')['corresponding field name 1']
        self.mapping_dic_alternative = self.mapping_sheet.set_index('2022 field name').T.to_dict('index')['corresponding field name 2']
        # After creating an empty data frame called procssed, the original data (df) crawled with 'InvestingDotcom collection code' is retrieved with the loop statement below.
        # standardize
        self.processed = pd.DataFrame(columns = self.mapping_dic.keys())
        
    # Function to extract currency identification code from bs_unit 
    def currency_extract(self, x) :
        world_currency = ['HKD', 'EUR', 'AUD', 'KRW', 'NZD', 'PHP', 'USD', 'DKK', 'TRY', 'CAD', 'CLP','INR', 'EGP', 'NOK', 'MYR', 'MXN', 
                        'CHF', 'GBP', 'SGD', 'ARS', 'THB', 'JPY', 'CNY','IDR','VND', 'COP']
        for currency in world_currency : 
            try : 
                if str(currency) in x : return currency 
                else : continue
            except : return np.nan
        return np.nan    
    
    # Function to extract numeric units used in financial statements
    def unit_change(self,x) : 
        try : 
            if "Millions" in x : unit =  1000000
            elif "Billions" in x : unit = 1000000000
            else : unit = 0
        except : unit = x
        return unit
    
    # Function to extract ticker from company name
    def ticker_extract(self,x) : 
        ticker = re.findall('\(([^)]+)',x)
        ticker = ticker[0] # Since findall returns a list, only elements are extracted using indexing
        return ticker
    
    # Function to remove ticker from company name
    def ticker_delete(self,x) : 
        regex = "\(.*\)|\s-\s.*"
        name = re.sub(regex,'',x)
        return name
    
    # A function that reads the row's 'bs_time' column and creates a FY column for the fiscal year and a Quarter for the fiscal quarter.
    def ending_period_extract(self, x) : 
        try : 
            x = datetime.strptime(x, '%Y%d/%m')
            return x
        except : return 

    def report_type_generator(self,x) : 
        try : 
            if x.lower() == 'quarter' : return 'Q'
            else : return 'A'
        except : return np.nan

    # A function that returns the Hebron Star country code if an English country name is entered
    def hb_nation_code(self, x) : 
        hebronstar_code = {'japan':'JPN', 'hong-kong' : 'HKG','malaysia' : 'MYS','south-korea' : 'KOR',
                        'singapore':'SGP','thailand':'THA','vietnam':'VNM','indonesia':'IDN',
                        'india':'IND','united-states':'USA','spain':'ESP','switzerland':'CHE',
                        'australia':'AUS','united-kingdom':'GBR', 'france':'FRA','italy':'ITA',
                        'germany':'DEU','netherlands':'NLD','mexico':'MEX','colombia':'COL','canada':'CAN'}         
        try :
            result = hebronstar_code[x]
            return result
        except : 
            print("You entered the wrong country name. Please enter standardized country names.")
            print("ex) korea -> South Korea , hongkong -> Hong Kong, China")
            return
        

    # If you input the correspondence table address (mapping_sheet_dir) and the original data (df) crawled with the InvestingDotcom collection code as input,
    # Function to extract the desired data frame based on the correspondence table 

    def matching_process(self) : 
        for order, field in enumerate(self.processed.columns) : 
            if field == '현금및예치금액' : 
                try : 
                    self.processed.loc[:,field] = self.data.loc[:,self.processed[field]]
                    continue
                except : 
                    self.processed.loc[:,field] = self.data.loc[:,self.mapping_dic_alternative[field]]
                    continue

            try : self.processed.loc[:,field] = self.data.loc[:,self.mapping_dic[field]]
            except : pass
        data_size = len(self.processed['영문기업명'].unique())
        print(f'The original data collected from Investing.com was cleansed to extract financial statements for a total of {data_size} companies.')
        return self.processed
    

    
    # Put all the preceding methods in order in the cleanse method.
    # originalfile_dir : Source address of data crawled by 'investingDotcom collection code'
    # mapping_sheet_dir: mapping table address
    # Replace missing currency code: Option to select whether to replace missing currency code with the most frequent value.
    # Investing.com often does not show which currency unit is written in the financial statement.
    # When currency code missing replacement is not available in which currency unit, listed companies in that country look at the data collected so far
    # This is the code to imputation the null value with the currency unit that is mainly expressed.


import pandas as pd
from selenium.webdriver.common.by import By
import chromedriver_autoinstaller
from selenium import webdriver
from selenium.webdriver.chrome.service import Service
from webdriver_manager.chrome import ChromeDriverManager

class information:
    def __init__(self):
        self.print_information()

    def print_information(self):
        print("""
        A description of the function is given below. \n
        The main class within the library is datos_extact. \n
        DriverSettings() is a function that launches the chrome driver. \n
        get_data() is a function that extracts and processes data. \n
        load() is a function that stores data.
        """)

class kemenperin_extract:
    def __init__(self, path):
        """
        Please put TableDefaultColumns Excel in the path.
        """
        self.DefaultDataFrame = pd.DataFrame(columns = list(pd.read_excel(path, sheet_name="일반_columns")["표준_영문컬럼명"]))

    def DriverSettings(self, Turn_off_warning = False, linux_mode = False) -> None:
        """
        This function sets the driver.
        If linux mode is set to True, collection is possible in the background.
        However, actions such as clicks cannot be taken.
        """
        if Turn_off_warning == True: self.TurnOffWarning()
        chrome_ver = chromedriver_autoinstaller.get_chrome_version().split('.')[0]  # Check chromedriver version
        chrome_options = webdriver.ChromeOptions()
        chrome_options.add_argument("--incognito") # incognito mode
        if linux_mode == True: chrome_options.add_argument("--headless") # Display mode without Linux GUI
        chrome_options.add_argument("--no-sandbox") # Prevent access to resources
        chrome_options.add_argument("--disable-setuid-sandbox") # Prevent chrome crashes
        chrome_options.add_argument("--disable-dev-shm-usage") # Prevent out of memory errors
        chrome_options.add_experimental_option('excludeSwitches', ['enable-logging'])
        try: # Chrome Driver
            self.driver = webdriver.Chrome(service=Service(ChromeDriverManager().install()), options=chrome_options)   
        except:
            chromedriver_autoinstaller.install(True)
            self.driver = webdriver.Chrome(service=Service(ChromeDriverManager().install()), options=chrome_options)
        # Prevent WebDruverException Error Designate as an existing driver version

    def get_data(self, city = 51, page_num = 1):
        """
        Function to extract data \n
        For the city, access KEMENPERIN and check the city number. \n
        Please set page_num.
        """
        self.driver.get(f"https://kemenperin.go.id/direktori-perusahaan?what=&prov={city}&hal={page_num}")
        tbody = self.driver.find_element(By.TAG_NAME, "tbody")
        tr = tbody.find_elements(By.TAG_NAME, "tr")
        AppendDict = {
            "hb_ntn_cd" : "IDN",
            "acplc_lngg_ntn_nm" : "Indonesia",
            "engls_ntn_nm" : "Indonesia",
            "ntn_lngg_cd_val" : "IDN",
            "acplc_lngg_lngg_nm" : "Indonesia",
            "engls_lngg_nm" : "Indonesia",
            "acplc_lngg_entrp_nm" : None,
            "engls_entrp_nm" : None,
            "acplc_lngg_oln_intrd_cont" : None,
            "acplc_lngg_entrp_intrd_cont" : None,
            "engls_oln_intrd_cont" : None,
            "entrp_rprsn_tlno" : None,
            "acplc_lngg_entrp_addr" : None,
            "acplc_lngg_entrp_dtadd" : None,
            "engls_entrp_addr" : None,
            "engls_entrp_dtadd" : None
        }
        for td in tr:
            d = td.find_elements(By.TAG_NAME, "td")
            number = d[0].text
            information = d[1].text.split("\n")
            product = d[2].text
            company_name = information[0]
            company_address = information[1]
            company_tel = information[2].replace("Telp." , "")
            AppendDict["acplc_lngg_entrp_nm"] = company_name
            AppendDict["engls_entrp_nm"] = company_name
            AppendDict["acplc_lngg_oln_intrd_cont"] = information
            AppendDict["engls_oln_intrd_cont"] = information
            AppendDict["entrp_rprsn_tlno"] = company_tel
            AppendDict["acplc_lngg_entrp_addr"] = company_address
            AppendDict["acplc_lngg_entrp_dtadd"] = company_address
            AppendDict["engls_entrp_addr"] = company_address
            AppendDict["engls_entrp_dtadd"] = company_address
            AppendDataFrame = pd.DataFrame(AppendDict)
            self.DefaultDataFrame = self.DefaultDataFrame.append(AppendDict, ignore_index= True)

    def load(self):
        self.DefaultDataFrame.to_excel("hb_tb_idn_egnin_m.xlsx", index=False)


import pandas as pd
import chromedriver_autoinstaller
import warnings
from selenium import webdriver
from tqdm import tqdm
from selenium.webdriver.common.by import By
from selenium.webdriver.chrome.service import Service
from webdriver_manager.chrome import ChromeDriverManager
from selenium import webdriver

warnings.simplefilter(action='ignore', category=FutureWarning)
warnings.filterwarnings("ignore", category=DeprecationWarning) 

class information:
    def __init__(self):
        self.print_information = """
        The function is described below. \n
        The main class in the library is opencorporates_extract. \n
        DriverSettings() is a function that sets the driver. \n
        """

    def __repr__(self) -> str:
        return str(self.print_information)
        


class yellow_extract:
    def __init__(self):
        self.url = lambda x: x
        self.country_dict = {
            "파키스탄" : "https://www.businesslist.pk/",
            "아르헨티나" : "https://www.arempresas.com/",
            "칠레" : "https://www.yelu.cl/",
            "브라질" : "https://www.brazilyello.com/",
            "카자흐스탄" : "https://www.kazakhstanyp.com/",
            "벨라루스" : "https://www.yelo.by/",
            "르완다" : "https://www.rwandayp.com/",
            "탄자니아" : "https://www.tanzapages.com/", 
            "우간다" : "https://www.yellow.ug/",
            "남아공" : "https://www.yellosa.co.za/",
            "일본" : "https://www.japanyello.com/",
            "홍콩" : "https://www.yelo.hk/",
            "말레이시아" : "https://www.businesslist.my/",
            "싱가포르" : "https://www.yelu.sg/",
            "태국" : "https://www.thaiyello.com/",
            "베트남" : "https://www.vietnamyello.com/",
            "인도네시아" : "https://www.indonesiayp.com/",
            "인도" : "https://www.yelu.in/",
            "멕시코" : "https://www.yelo.com.mx/",
            "네덜란드" : "https://www.yelu.nl/",
            "호주" : "https://www.australiayp.com/",
            "스위스" : "https://www.swissyello.com/"
        }
    def __repr__(self):
        return str(self.country_dict.keys())

    def DriverSettings(self, Turn_off_warning = False, linux_mode = False) -> None:
        """
        This function sets the driver.
        If linux mode is set to True, collection is possible in the background.
        However, actions such as clicks cannot be taken.
        """
        if Turn_off_warning == True: self.TurnOffWarning()
        chrome_ver = chromedriver_autoinstaller.get_chrome_version().split('.')[0]  # Check chromedriver version
        chrome_options = webdriver.ChromeOptions()
        chrome_options.add_argument("--incognito") # incognito mode
        if linux_mode == True: chrome_options.add_argument("--headless") # Display mode without Linux GUI
        chrome_options.add_argument("--no-sandbox") # Prevent access to resources
        chrome_options.add_argument("--disable-setuid-sandbox") # Prevent chrome crashes
        chrome_options.add_argument("--disable-dev-shm-usage") # Prevent out of memory errors
        chrome_options.add_experimental_option('excludeSwitches', ['enable-logging'])
        try: # Chrome Driver
            self.driver = webdriver.Chrome(service=Service(ChromeDriverManager().install()), options=chrome_options)   
        except:
            chromedriver_autoinstaller.install(True)
            self.driver = webdriver.Chrome(service=Service(ChromeDriverManager().install()), options=chrome_options)
        # Prevent WebDruverException Error Designate as an existing driver version

    def moment_save(self, number, country_name):
        self.companies_info.to_csv(f"{country_name}_{number}.csv", index=False)
        return f"{country_name} is passed"

    def extract(self, country_name, moment_stop = False):
        self.companies_info = pd.DataFrame()
        self.driver.get(self.url(self.country_dict[country_name]))

        location_list = self.driver.find_element(By.XPATH, '/html/body/section[2]/ul')
        locations = location_list.find_elements(By.TAG_NAME, 'a')
        self.location_links = [location.get_attribute("href") for location in locations if "location" in location.get_attribute("href")]

        for link in set(self.location_links):
            self.driver.get(link)
            find_max_page_number = self.driver.find_element(By.XPATH, '//*[@id="listings"]')
            max_page_number = max(int(i.text) for i in find_max_page_number.find_elements(By.TAG_NAME, 'a') if i.get_attribute("class") == "pages_no")
            
            for idx in tqdm(range(1, max_page_number + 1)):
                if idx > 1: self.driver.get(f"{link}/{idx}")
                
                if moment_stop == True and idx % 100 == 0:
                    self.moment_save(idx, country_name)
                    break
                    
                href_list = self.driver.find_element(By.XPATH, '//*[@id="listings"]')
                company_links = [href.get_attribute("href") for href in href_list.find_elements(By.TAG_NAME, "a") if "company" in href.get_attribute("href") and "reviews" not in href.get_attribute("href")]

                for company_link in set(company_links):
                    self.driver.get(company_link)
                    left_list = self.driver.find_element(By.XPATH, '//*[@id="left"]')
                    div_list = left_list.find_elements(By.TAG_NAME, "div")
                    
                    try:
                        info_texts = [div.text for div in div_list if div.get_attribute('class') == ['info']]
                        if not info_texts:
                            continue
                        company_name = info_texts[0].split("\n")[1]
                        append_df = pd.DataFrame({company_name: info_texts[1:]})
                        self.companies_info = pd.concat([self.companies_info, append_df], axis=1)
                        print(f"appended {company_name}")
                    except Exception as e:
                        print(f"Error while processing {company_link}: {e}")
                        continue
        self.driver.close()

        return self.companies_info





import os
import pandas as pd
from tqdm import tqdm

class information:
    def __init__(self):
        self.print_information()

    def print_information(self):
        print("""
        A description of the function is given below. \n
        The main class within the library is idx_extact. \n
        make_Avaible() is a function that makes a data frame available. \n
        Add_On() is a function that creates data. \n
        transform() is a function that processes data.
        """)

class idx_extract:
    def __init__(self, dataframe):
        self.df = dataframe

    def MakeAvaible(self):
        try:
            self.df = self.df[["Unnamed: 3", "Unnamed: 1"]].T
            self.df = df.rename(columns=self.df.iloc[0])
            self.df = self.df.drop(index="Unnamed: 3")
            self.df = df.fillna(0)
        except:
            df = df[["Unnamed: 2", "Unnamed: 1"]].T
            df = df.rename(columns=df.iloc[0])
            df = df.drop(index="Unnamed: 2")
            df = df.fillna(0)
        return df

    def Add_On(self, mapping_path):
        self.mapping_sheet = pd.read_excel(mapping_path)
        self.mapping_dic = self.mapping_sheet.set_index('채워야할 테이블 필드명').T.to_dict('index')['idx']
        self.mapping_dic_2 = self.mapping_sheet.set_index('idx').T.to_dict('index')['채워야할 테이블 필드명']
        self.processed = pd.DataFrame(columns = self.mapping_dic.keys())
        self.Balance = self.mapping_sheet["Balance Sheet"]
        self.Income = self.mapping_sheet["Income Statement"]
        self.Cash = self.mapping_sheet["Cashflow"]
        self.General = self.mapping_sheet["General"]
        self.Balance = self.Balance.dropna()
        self.Income = self.Income.dropna()
        self.Cash = self.Cash.dropna()
        self.General = self.General.dropna()

    def transform(self, Path):
        FilePath = os.listdir(Path)
        for File in tqdm(FilePath):
            df_general = pd.read_excel(Path+File, sheet_name=1)
            df_balance = pd.read_excel(Path+File, sheet_name=2)
            df_income = pd.read_excel(Path+File, sheet_name=3)
            df_cash = pd.read_excel(Path+File, sheet_name=6)
            
            ticker_name = str(File).split("-")[3]
            quater = str(File).split("-")[2]
            success_columns = []
            success_index = []

            success_columns.append('주식시장코드')
            success_columns.append('현지언어주식시장명')
            success_columns.append('영문주식시장명')
            success_columns.append('헤브론스타국가코드')
            success_columns.append('통화구분코드')
            success_columns.append('회계연도')
            success_columns.append('보고서종류코드')
            success_columns.append('결산일자')


            success_index.append('IDX')
            success_index.append('IDX')
            success_index.append('IDX Stock')
            success_index.append('IDN')
            success_index.append('IDR')
            success_index.append(2022)
            success_index.append('Q')
            if quater == "I":
                success_index.append(20220228)
            elif quater == "II":
                success_index.append(20220530)
            elif quater == "III":
                success_index.append(20220830)



            for G_idx in self.General:
                try:
                    if self.MakeAvailble(df_general)[G_idx][0] == 0: pass
                    else:
                        success_columns.append(str(self.mapping_dic_2[G_idx]))
                        success_index.append(self.MakeAvailble(df_general)[G_idx][0])
                except: pass

            for B_idx in self.Balance:
                try:
                    if self.MakeAvailble(df_balance)[B_idx][0] == 0: pass
                    else:
                        success_columns.append(str(self.mapping_dic_2[B_idx]))
                        success_index.append(self.MakeAvailble(df_balance)[B_idx][0])
                except: pass
            
            for I_idx in self.Income:
                try:
                    if self.MakeAvailble(df_income)[I_idx][0] == 0: pass
                    else: 
                        success_columns.append(str(self.mapping_dic_2[I_idx]))
                        success_index.append(self.MakeAvailble(df_income)[I_idx][0])
                except: pass

            for C_idx in self.Cash:
                try: 
                    if self.MakeAvailble(df_cash)[C_idx][0] == 0: pass
                    else: 
                        success_columns.append(str(self.mapping_dic_2[C_idx]))
                        success_index.append(self.MakeAvailble(df_cash)[C_idx][0])
                except: pass

            if len(success_columns) > 0:     
                complete_df = pd.DataFrame([tuple(success_index)], columns=success_columns)
                processed = processed.append(complete_df, ignore_index=False)

        processed.to_excel("complete_indonesia_2022.xlsx")


import socket
from _thread import *
import pandas as pd
import time
from bs4 import BeautifulSoup
from urllib.request import Request, urlopen
from tqdm import tqdm
import urllib.request

class information:
    def __init__(self):
        self.print_information()

    def print_information(self):
        print("""
        The function is described below. \n
        The main class in the library is extract().\n
        extract() collects data from wsj.\n
        collect() imports only OTC companies of the data collected.
        """)

class wsj:
    def __init__(self, usa_indexes):
        """
        please input usa_indexes excel file \n
        usa_indexes in wsj homepage
        """
        self.df = pd.read_excel(usa_indexes)
        self.OTC_df = self.df[self.df["Exchange"] == "OOTC"]

        self.lst = ["balance-sheet", "cash-flow", "income-statement"]
        self.cf_df = pd.DataFrame()
        self.bs_df = pd.DataFrame()
        self.is_df = pd.DataFrame()
        self.failed_lst = []
        self.cnt = 0
        self.number = 0

    def extract(self, symbols_name, type, name):
        url = f"https://www.wsj.com/market-data/quotes/{symbols_name}/financials/annual/{idx}"
        try: response = urlopen(url, timeout=5)
        except urllib.error.URLError as e:
            if isinstance(e.reason, socket.timeout):
                print("time out.")
                time.sleep(3)
                self.extract(symbols_name, type, name)
        try:
            urlTicker = Request(url, headers={'User-Agent': 'Mozilla/5.0'})
            readTicker = urlopen(urlTicker).read()
            soup = BeautifulSoup(readTicker, 'lxml')
            table = soup.find_all('table')
        except: return
        try: df = pd.read_html(str(table))[0]
        except: return
        # df["name"] = name
        if type == "balance-sheet":
            try: df.to_excel(f"./balance_sheet/bs_df_{symbols_name}.xlsx", index=False)
            except: 
                self.failed_lst.append(symbols_name)
                print(f"{name} is Failed")
            # self.bs_df = self.bs_df.append(df)
        elif type == "cash-flow": 
            try: df.to_excel(f"./cash_flow/cs_df_{symbols_name}.xlsx", index=False)
            except:
                self.failed_lst.append(symbols_name)
                print(f"{name} is Failed")
            # self.cf_df = self.cf_df.append(df)
        elif type == "income-statement": 
            try: df.to_excel(f"./income_statement/is_df_{symbols_name}.xlsx", index=False)
            except:
                self.failed_lst.append(symbols_name)
                print(f"{name} is Failed")
            # self.is_df = self.is_df.append(df)
        print(f"data extracted. {name}")

    def collect(self):
        for name in tqdm(list(self.OTC_df["Name"])[990:]):
            symbols_name = name.split("(")[-1].replace(")","")
            for type in self.lst:
                self.extract(symbols_name, type, name)


import pandas as pd
from tqdm import tqdm

class information:
    def __init__(self):
        self.print_information()

    def print_information(self):
        print("""
        A description of the function is given below. \n
        The main class within the library is MARCAP. \n
        install() is a function that informs the marcap data github address. \n
        collect() is a function that extracts data.
        """)

class MARCAP:

    def __init__(self, ticker_lst) -> list:
        """
        please input ticker_lst
        """
        self.exist_lst = []
        self.ticker_lst = ticker_lst
    
    def install():
        return '!git clone "https://github.com/FinanceData/marcap.git" marcap'
    
    def collect(self):
        from marcap import marcap_data
        for name in tqdm(self.ticker_lst):
            try:
                df = marcap_data('2018-01-01', '2023-01-01', code='005930') 
                df = df.reset_index()
                df.to_csv("marcap_data/" + name + "_5yr.csv", index=False)
            except:
                self.exist_lst.append(name)
                print(name + " data does not exist")
                pass


import pandas as pd
import chromedriver_autoinstaller
import re
from bs4 import BeautifulSoup
from selenium import webdriver
from tqdm import tqdm
from selenium.webdriver.common.by import By
from selenium.webdriver.chrome.service import Service
from webdriver_manager.chrome import ChromeDriverManager

class information:
    def __init__(self):
        self.print_information()

    def print_information(self):
        print("""
        A description of the function is given below. \n
        The main class within the library is BIZIN. \n
        In the case of Asian countries, the url is different, so you need to set it. \n
        DriverSettings() is a Selenium Chrome driver settings function. \n
        area() is a function that collects information on companies in the country. \n
        collect() is a function that collects data from the BIZIN site. \n
        """)

class country_name:
    def __init__(self):
        self.print_information()

    def print_information(self):
        self.country_name = {"영국" : "gb", "독일" : "de", "스위스" : "ch", "호주" : "at",
                            "이란" : "ie", "벨기에" : "be","스페인" : "es","포르투칼" : "pt","프랑스" : "fr",
                            "이탈리아" : "it","덴마크" : "dk","네덜란드" : "nl","노르웨이" : "no","핀란드" : "fi","스웨덴" : "se",
                            "라트비아" : "lv","에스토니아" : "ee","폴란드" : "pl","벨라루스" : "by",
                            "우크라이나" : "ua","룩셈부르크" : "lu","그리스" : "gr","헝가리" : "hu","중국" : "china",
                            "인도" : "india","인도네시아" : "indonesia","대한민국" : "southkorea","일본" : "japan",
                            "싱가포르" : "singapore","말레이시아" : "malaysia","홍콩" : "hongkong","태국" : "tahiland",
                            "베트남" : "vietnam","필리핀" : "philippines","타이완" : "taiwan","파키스탄" : "pakistan","사우디" : "saudiarabia"}
        print(f"""
        The description of the country that can be put in the name is as follows. \n
        {self.country_name}
        """)
        

class BIZIN:
    def __init__(self, name, asia = False):
        self.url = f"https://{name}.bizin.eu/"
        if asia: self.url = f"https://{name}.bizin.asia/"
    
    def DriverSettings(self, Turn_off_warning = False, linux_mode = False) -> None:
        """
        This function sets the driver.
        If linux mode is set to True, collection is possible in the background.
        However, actions such as clicks cannot be taken.
        """
        if Turn_off_warning == True: self.TurnOffWarning()
        chrome_ver = chromedriver_autoinstaller.get_chrome_version().split('.')[0]  # Check chromedriver version
        chrome_options = webdriver.ChromeOptions()
        chrome_options.add_argument("--incognito") # incognito mode
        if linux_mode == True: chrome_options.add_argument("--headless") # Display mode without Linux GUI
        chrome_options.add_argument("--no-sandbox") # Prevent access to resources
        chrome_options.add_argument("--disable-setuid-sandbox") # Prevent chrome crashes
        chrome_options.add_argument("--disable-dev-shm-usage") # Prevent out of memory errors
        chrome_options.add_experimental_option('excludeSwitches', ['enable-logging'])
        try: # Chrome Driver
            self.driver = webdriver.Chrome(service=Service(ChromeDriverManager().install()), options=chrome_options)   
        except:
            chromedriver_autoinstaller.install(True)
            self.driver = webdriver.Chrome(service=Service(ChromeDriverManager().install()), options=chrome_options)
        # Prevent WebDruverException Error Designate as an existing driver version


    def area(self):
        self.driver.get(self.url)
        self.areas_lst = {}
        self.areas_href = {}
        areas = self.driver.find_element(By.XPATH, '//*[@id="main_categories_dk"]')
        for area in areas.find_elements(By.TAG_NAME, "div"):
            try:
                name = area.text
                pattern = r'\d+'
                numbers = re.findall(pattern, name)
                numbers = [int(n) for n in numbers][0]
                replace_pattern = r'\s*\(\d+\)'
                replace_name = re.sub(replace_pattern, '', name)
                cnt = int(numbers) 
                self.areas_lst[replace_name] = cnt
                href = area.find_element(By.TAG_NAME, "a").get_attribute("href")
                self.areas_href[replace_name] = href
            except:
                print(f"fail {area.text}")
                pass
        self.max_area = max(self.areas_lst, key=self.areas_lst.get)


    def collect(self):
        first = 0
        for num in range(1, self.areas_lst[self.max_area]//20):
            self.driver.get(f"{self.areas_href[self.max_area]}?p={num}")
            try: org_lst = self.driver.find_element(By.XPATH, "/html/body/div/main/div/div[6]/div[1]") 
            except: 
                try: org_lst = self.driver.find_element(By.XPATH, "/html/body/div[2]/main/div/div[6]/div[1]") 
                except: 
                    try: org_lst = self.driver.find_element(By.XPATH, "/html/body/div[3]/main/div/div[6]/div[1]")
                    except: continue
            self.href_lst = {}
            for href in org_lst.find_elements(By.TAG_NAME, "div"):
                try:
                    self.href_lst[(href.find_element(By.TAG_NAME, "a").text)] = href.find_element(By.TAG_NAME, "a").get_attribute("href")
                except: pass
            self.href_lst = {key:value for key, value in self.href_lst.items() if "bizin" in value}
            for name, idx in tqdm(self.href_lst.items()):
                try:
                    self.driver.get(idx)
                    company_name = name
                    page = self.driver.page_source
                    soup = BeautifulSoup(page, 'html.parser')
                    if first == 0:
                        self.df = pd.read_html(str(soup.find('table')))[0]
                        columns = self.df.transpose().iloc[0]
                        self.df = self.df.T
                        self.df.columns = columns
                        self.df = self.df.drop(0, axis=0)
                        self.df["name"] = company_name
                        first += 1
                    else:
                        self.append_df = pd.read_html(str(soup.find('table')))[0]
                        self.append_df_columns = self.append_df.transpose().iloc[0]
                        self.append_df = self.append_df.T
                        self.append_df.columns = list(self.append_df_columns)
                        self.append_df = self.append_df.drop(0, axis=0)
                        self.append_df["name"] = company_name
                        self.df = self.df.append(self.append_df)
                except:
                    print("Failed")



import pandas as pd
import chromedriver_autoinstaller
from selenium import webdriver
from tqdm import tqdm
from selenium.webdriver.common.by import By
from selenium.webdriver.chrome.service import Service
from webdriver_manager.chrome import ChromeDriverManager

class investing_news_extract:
    def __init__(self):
        pass

    def set_columns(self):
        """
        Column setting for news
        """
        self.news_append_columns = ["entrp_reltn_tdngs_url", "entrp_reltn_tdngs_subjc", "entrp_reltn_tdngs_dtl_cont"]
        self.new_news_columns = ["industry_reltn_tdngs_url", "industry_reltn_tdngs_subjc", "industry_reltn_tdngs_dtl_cont"]

    def DriverSettings(self, Turn_off_warning = False, linux_mode = False) -> None:
        """
        This function sets the driver.
        If linux mode is set to True, collection is possible in the background.
        However, actions such as clicks cannot be taken.
        """
        if Turn_off_warning == True: self.TurnOffWarning()
        chrome_ver = chromedriver_autoinstaller.get_chrome_version().split('.')[0]  # Check chromedriver version
        chrome_options = webdriver.ChromeOptions()
        chrome_options.add_argument("--incognito") # incognito mode
        if linux_mode == True: chrome_options.add_argument("--headless") # Display mode without Linux GUI
        chrome_options.add_argument("--no-sandbox") # Prevent access to resources
        chrome_options.add_argument("--disable-setuid-sandbox") # Prevent chrome crashes
        chrome_options.add_argument("--disable-dev-shm-usage") # Prevent out of memory errors
        chrome_options.add_experimental_option('excludeSwitches', ['enable-logging'])
        try: # Chrome Driver
            self.driver = webdriver.Chrome(service=Service(ChromeDriverManager().install()), options=chrome_options)   
        except:
            chromedriver_autoinstaller.install(True)
            self.driver = webdriver.Chrome(service=Service(ChromeDriverManager().install()), options=chrome_options)
        # Prevent WebDruverException Error Designate as an existing driver version

    def input_dataframe(self, df) -> pd.DataFrame:
        """
        Set the data frame to be collected
        """
        self.df = df

    def get_information(self):
        """
        Collect news from Investing based on the listing code of the data frame.
        """
        symbol = list(set(self.df["lstng_cd"]))
        self.news_dict = {}
        for idx in tqdm(symbol):
            html = f"https://www.investing.com/search/?q={idx}&tab=news"
            self.driver.get(html)
            article_lst = []
            for dix in self.driver.find_elements(By.XPATH, "//div[contains(@class, 'articleItem')]"):
                article = dix.find_element(By.TAG_NAME, 'a').get_attribute("href")
                if "news/" in article: article_lst.append(article)
            self.news_dict[idx] = article_lst 

    def change_length(self):
        max_len = max(map(len, self.news_dict.values()))

        # fit all lists to the length of the longest list
        for key, value in self.news_dict.items():
            if len(value) < max_len:
                self.news_dict[key] = value + [None] * (max_len - len(value))
        self.df = pd.DataFrame(self.news_dict)

    def collect(self):
        """
        Collect news information by accessing the url address collected from investing
        """
        self.base_df = pd.DataFrame()
        for col in tqdm(list(self.df.columns)):
            symbol = col
            for length in range(len(self.df[col])):
                try:
                    url = str(self.df[col][length])
                    if not url == "nan":
                        self.driver.get(url)
                        article_header = self.driver.find_element(By.CLASS_NAME, 'articleHeader').text
                        article_page = self.driver.find_element(By.XPATH, "//div[contains(@class, 'articlePage')]")
                        article_date = self.driver.find_element(By.CLASS_NAME, 'contentSectionDetails').text
                        articles = []
                        for article in article_page.find_elements(By.TAG_NAME, 'p'):
                            articles.append(article.text)
                        article_joined = " ".join(articles)
                    infor_dict = {
                        "symbol" : symbol,
                        "article_header" : article_header,
                        "article_date" : article_date,
                        "article" : article_joined,
                        "article_url" : url
                    }
                    append_df = pd.DataFrame(infor_dict, index=[0])
                    self.base_df = self.base_df.append(append_df, ignore_index=True)
                except: pass

    def merge_data(self, path):
        """
        Combining data collected from url addresses
        """
        self.df = self.df.drop(columns=["Unnamed : 0"])
        self.df = self.df.dropna()
        for idx, jdx in zip(self.news_append_columns, self.new_news_columns):
            self.df[jdx] = self.df[idx]
        self.df = self.df.drop(columns="article_date")
        for idx in tqdm(range(len(self.base_df))):
            symbol = self.base_df["lstng_cd"][idx]
            append_df = self.df[self.df["lstng_cd"] == symbol]
            if not append_df.empty:
                if str(self.base_df.loc[idx, list(append_df.columns)]["industry_reltn_tdngs_url"]) == "nan":
                    self.base_df.loc[idx, list(append_df.columns)] = append_df.values[0]
                else:
                    print(str(self.base_df.loc[idx, list(append_df.columns)]["industry_reltn_tdngs_url"]))
        self.base_df.to_excel(path, index=False)


import pandas as pd
import chromedriver_autoinstaller
from selenium import webdriver
from tqdm import tqdm
from selenium.webdriver.common.by import By
from selenium.webdriver.chrome.service import Service
from webdriver_manager.chrome import ChromeDriverManager
from selenium import webdriver
from selenium.webdriver.support import expected_conditions as EC

class information:
    def __init__(self):
        self.print_information = """
        The function is described below. \n
        The main class in the library is cybo_extract. \n
        DriverSettings() is a function that sets the driver. \n
        collect() is collect data.
        """

class cybo_extract:
    def __init__(self):
        self.url = lambda x: f"https://www.cybo.com/UA/kiev-kiev/hotels-&-travel/?p={x}"

    def DriverSettings(self, Turn_off_warning = False, linux_mode = False) -> None:
        """
        This function sets the driver.
        If linux mode is set to True, collection is possible in the background.
        However, actions such as clicks cannot be taken.
        """
        if Turn_off_warning == True: self.TurnOffWarning()
        chrome_ver = chromedriver_autoinstaller.get_chrome_version().split('.')[0]  # Check chromedriver version
        chrome_options = webdriver.ChromeOptions()
        chrome_options.add_argument("--incognito") # incognito mode
        if linux_mode == True: chrome_options.add_argument("--headless") # Display mode without Linux GUI
        chrome_options.add_argument("--no-sandbox") # Prevent access to resources
        chrome_options.add_argument("--disable-setuid-sandbox") # Prevent chrome crashes
        chrome_options.add_argument("--disable-dev-shm-usage") # Prevent out of memory errors
        chrome_options.add_experimental_option('excludeSwitches', ['enable-logging'])
        try: # Chrome Driver
            self.driver = webdriver.Chrome(service=Service(ChromeDriverManager().install()), options=chrome_options)   
        except:
            chromedriver_autoinstaller.install(True)
            self.driver = webdriver.Chrome(service=Service(ChromeDriverManager().install()), options=chrome_options)
        # Prevent WebDruverException Error Designate as an existing driver version

    def collect(self):
        link_lst = []
        map_lst = []
        for idx in range(1, 11):
            self.driver.get(self.url(idx))
            div_lst = self.driver.find_element(By.XPATH, '//*[@id="bottom"]')
            href_lst = div_lst.find_elements(By.TAG_NAME, 'a')
            for href in href_lst:
                link = href.get_attribute("href")
                if "UA-biz" in link and link not in link_lst: link_lst.append(link)
                elif "r/biz" in link and link not in map_lst: map_lst.append(link)
        self.company_df = pd.DataFrame()
        for link in tqdm(link_lst):
            self.driver.get(link)
            company_name = self.driver.find_element(By.XPATH, '//*[@id="content"]/div[2]/div[1]/div/div/div/h1').text
            company_address = self.driver.find_element(By.XPATH, '//*[@id="content"]/div[2]/div[1]/div/div/div/div[2]').text
            left_content = self.driver.find_element(By.XPATH, '//*[@id="left-contact"]').text
            lst_split = left_content.split("\n")
            for idx in range(len(lst_split)):
                if lst_split[idx-1] == "Phone": phone_number = lst_split[idx]
                elif lst_split[idx-1] == "Website ": company_url = lst_split[idx]
            about = self.driver.find_element(By.XPATH, '//*[@id="right_col"]/div[2]/div/div/div[1]/div').text
            ISIC_code = self.driver.find_element(By.XPATH, '//*[@id="right_col"]/div[2]/div/div/div[2]/div[2]/span[2]/span').text
            categories = self.driver.find_element(By.XPATH, '//*[@id="right_col"]/div[2]/div/div/div[2]/div[1]').text
            company_dict = {"company_name" : company_name,
                        "company_address" : company_address,
                        "phone_number" : phone_number,
                        "company_url" : company_url,
                        "about" : about,
                        "ISIC_code" : ISIC_code,
                        "categories" : categories}
            if self.company_df.empty : self.company_df = pd.DataFrame(company_dict, index=[0])
            else: self.company_df = self.company_df.append(company_dict, ignore_index=True)
        self.company_df.to_excel("우크라이나.xlsx", index = False)


import pandas as pd

class information:
    def __init__(self):
        self.print_information()

    def print_information(self):
        print("""
        A description of the function is given below. \n
        The main class within the library is datos_extact. \n
        make() is a function that processes data. \n
        load() is a function that stores data.
        """)

class esercizi_extract:
    def __init__(self, path):
        """
        Please put TableDefaultColumns Excel in the path.
        """
        self.DefaultDataFrame = pd.DataFrame(columns = list(pd.read_excel(path, sheet_name="일반_columns")["표준_영문컬럼명"]))

    def make(self, path):
        """
        Function to standardize data
        Put the excel downloaded from Datos in the path.
        """
        df = pd.read_excel(path)
        AppendDict = {
            "hb_ntn_cd" : "ITA",
            "acplc_lngg_ntn_nm" : "Italy",
            "engls_ntn_nm" : "Italy",
            "ntn_lngg_cd_val" : "ITA",
            "acplc_lngg_lngg_nm" : "Italy",
            "engls_lngg_nm" : "Italy",
            "acplc_lngg_entrp_nm" : None,
            "engls_entrp_nm" : None,
            "acplc_lngg_oln_intrd_cont" : None,
            "acplc_lngg_entrp_intrd_cont" : None,
            "engls_oln_intrd_cont" : None,
            "engls_entrp_intrd_cont" : None,
            "entrp_rprsn_tlno" : None,
            "acplc_lngg_entrp_addr" : None,
            "acplc_lngg_entrp_dtadd" : None, 
            "engls_entrp_addr" : None,
            "engls_entrp_dtadd" : None,
            "acplc_lngg_ceo_nm" : None,
            "engls_ceo_nm" : None,
            "fndtn_dt" : None 
        }
        AppendDict["acplc_lngg_entrp_nm"] = list(df["COMPANY NAME"])
        AppendDict["engls_entrp_nm"] = list(df["COMPANY NAME"])
        AppendDict["acplc_lngg_ceo_nm"] = list(df["LEG. REPRESENTATIVE"])
        AppendDict["engls_ceo_nm"] = list(df["LEG. REPRESENTATIVE"])
        AppendDict["fndtn_dt"] = list(df["COMMUNICATION DATE"])
        AppendDict["acplc_lngg_entrp_addr"] = list(df["LOCATION"])
        AppendDict["engls_entrp_addr"] = list(df["LOCATION"])
        AppendDataFrame = pd.DataFrame(AppendDict)
        self.DefaultDataFrame = self.DefaultDataFrame.append(AppendDataFrame, ignore_index=True)

    def load(self):
        """
        데이터를 저장하는 함수
        """
        self.DefaultDataFrame.to_excel("tb_hb_ita_egnin_m.xlsx", index = False)


import pickle
import base64
from cryptography.fernet import Fernet
from cryptography.hazmat.primitives import hashes
from cryptography.hazmat.primitives.kdf.pbkdf2 import PBKDF2HMAC
import os
import sqlalchemy

def generate_key(password, salt):
    kdf = PBKDF2HMAC(
        algorithm=hashes.SHA256(),
        length=32,
        salt=salt,
        iterations=100000,
    )
    return base64.urlsafe_b64encode(kdf.derive(password))

module_path = os.path.abspath(os.path.dirname(__file__))
enc_file_path = os.path.join(module_path, 'processing_constants.enc')

with open(enc_file_path, 'rb') as file:
    salt = file.read(16)
    encrypted_data = file.read()

password = input("input your password").encode()

key = generate_key(password, salt)

# create encryption object
cipher_suite = Fernet(key)

# decryption
decrypted_data = cipher_suite.decrypt(encrypted_data)

# Load data from Pickle
loaded_data = pickle.loads(decrypted_data)

country = loaded_data["country"]
dtypesql_finan = loaded_data["dtypesql_finan"]
definition_finan = loaded_data["definition_finan"]
dtypesql_info = loaded_data["dtypesql_info"]
definition_info = loaded_data["definition_info"]
stock_name = loaded_data["stock_name"]
TableDefaultColumns = loaded_data["TableDefaultColumns"]
TableDefaultColumns_info = loaded_data["TableDefaultColumns_info"]
TableDefault = loaded_data["TableDefault"]
definition = loaded_data["definition"]
ColumnsDict = loaded_data["ColumnsDict"]
Financial = loaded_data["Financial"]
Month = loaded_data["Month"]
NumericList = loaded_data["NumericList"]
MonthDict = loaded_data["MonthDict"]
stock_info = loaded_data["stock_info"]


import pandas as pd
import time
import psycopg2
import chromedriver_autoinstaller
from webdriver_manager.chrome import ChromeDriverManager
from selenium import webdriver
from tqdm import tqdm
from selenium.webdriver.common.by import By
from selenium.webdriver.chrome.service import Service
from selenium.webdriver.support import expected_conditions as EC

class information:
    def __init__(self):
        self.print_information()

    def print_information(self):
        print("""
        The SHAREOUTSTANDING library collects market cap data. \n
        DriverSettings() is a Selenium Chrome driver settings function. \n
        get_company() is a function that retrieves a ticker from our US company database and stores its value. \n
        collect() is a function that collects data from shareoutstanding sites. \n
        """)

class SHAREOUTSTANDING:
    def __init__(self):
        self.url = "https://www.sharesoutstandinghistory.com/"

    def DriverSettings(self, Turn_off_warning = False, linux_mode = False) -> None:
        """
        This function sets the driver.
        If linux mode is set to True, collection is possible in the background.
        However, actions such as clicks cannot be taken.
        """
        if Turn_off_warning == True: self.TurnOffWarning()
        chrome_ver = chromedriver_autoinstaller.get_chrome_version().split('.')[0]  # Check chromedriver version
        chrome_options = webdriver.ChromeOptions()
        chrome_options.add_argument("--incognito") # incognito mode
        if linux_mode == True: chrome_options.add_argument("--headless") # Display mode without Linux GUI
        chrome_options.add_argument("--no-sandbox") # Prevent access to resources
        chrome_options.add_argument("--disable-setuid-sandbox") # Prevent chrome crashes
        chrome_options.add_argument("--disable-dev-shm-usage") # Prevent out of memory errors
        chrome_options.add_experimental_option('excludeSwitches', ['enable-logging'])
        try: # Chrome Driver
            self.driver = webdriver.Chrome(service=Service(ChromeDriverManager().install()), options=chrome_options)   
        except:
            chromedriver_autoinstaller.install(True)
            self.driver = webdriver.Chrome(service=Service(ChromeDriverManager().install()), options=chrome_options)
        # Prevent WebDruverException Error Designate as an existing driver version
        
    def get_company(self, host, database, user, password):
        conn = psycopg2.connect(
            host = host,
            database = database,
            user = user,
            password = password
        )
        query = "SELECT * FROM tb_hb_usa_plcfi_d"
        df = pd.read_sql(query, conn)

        self.company_lst = [i for i in list(df["lstng_cd"].unique()) if i is not None]


    def collect(self):
        self.driver.get(self.url)
        self.driver.maximize_window()
        df = pd.DataFrame()
        for idx in tqdm(self.company_lst):
            search_bar = self.driver.find_element(By.XPATH, '//*[@id="symbol"]')
            search_bar.clear()
            search_bar.send_keys(idx+"\n")
            time.sleep(2)
            try: self.driver.find_element(By.XPATH, '//*[@id="baltimore-button-no"]').click()
            except: pass
            try: 
                error_site = self.driver.find_element(By.XPATH, '/html/body/center/div[4]/div[2]/div[1]/h1').text
                if error_site == "404 File Not Found": continue
            except: pass
            time.sleep(2)
            table = self.driver.find_element(By.XPATH, '/html/body/center/div[4]/div[2]/div[2]/table[1]/tbody/tr[2]/td/center/table')
            table_html = table.get_attribute('outerHTML')
            append_df = pd.read_html(table_html)[0].T
            df = df.append(append_df, ignore_index=True)
        return df



import re
import pandas as pd
import random
import numpy as np
import warnings
import datetime
from tqdm import tqdm
from .constants import const
warnings.simplefilter(action='ignore', category=FutureWarning)
pd.set_option('mode.chained_assignment',  None)

class information:
    def __init__(self):
        self.print_information()

    def print_information(self):
        print("""
        A description of the function is given below. \n
        The library includes T (Transform) in the ETL process. \n
        The class that checks data in the database is Checker. \n
        When designating a class, enter the id, pw, ip, db of the database (postgresql), and the table name to be extracted. \n
        The read_excel() function loads xlsx and saves it as a data frame. \n
        The read_csv() function loads a csv and saves it as a data frame. \n
        The data_update() function inputs I or U when updating new data. \n
        The date_update() function inputs the date when new data is updated. \n
        The CheckDate() function is a function that standardizes the general data date of Investing.com \n
        The CheckLength() function checks the size of data and cuts it by the size \n
        The CheckVarchar() function checks the financial data size and inserts a new one if it is large \n
        The CheckNumeric() function checks a number in financial data \n
        -------------------------------------------------- ----------------\n
        The class that checks data from database is Analysis. \n
        The read_excel() function loads xlsx and saves it as a data frame. \n
        The read_csv() function loads a csv and saves it as a data frame. \n
        The Fail() function is a function that dictates erroneous data to put into a data frame \n
        The CheckDate_Duplicate() function is a function that checks the date check and duplicate check \n
        The CheckNumber() function checks whether a phone number is valid \n
         """)

class Checker:
    """
    A module that inspects data and replaces values as needed
    """
    def __init__(self):
        self.definition = const.definition
        self.ColumnsDict = const.ColumnsDict
        self.Financial = const.Financial
        self.Month = const.Month
        self.NumericList = const.NumericList
        self.MonthDict = const.MonthDict
    
    def fndtn_dt(self):
        """
        A function that normalizes fndtn_dt values in data to YYYYMMDD
        """
        for idx in range(len(self.df)):
            value = self.df["fndtn_dt"][idx]
            if str(type(value)) == "<class 'numpy.int64'>": pass
            elif str(type(value)) == "<class 'datetime.datetime'>": self.df["fndtn_dt"][idx] = value.strftime("%Y%m%d")
            elif str(type(value)) == "<class 'numpy.float64'>":
                if value.astype(int) > 21000000 or value.astype(int) < 18000000:
                    if str(value) == "nan": self.df["fndtn_dt"][idx] = ""
            else:
                self.df["fndtn_dt"][idx] = ""

    def read_excel(self, path):
        """
        Enter the excel address in path.
        """
        self.df = pd.read_excel(path)

    def read_csv(self, path):
        """
        Enter the csv address in path.
        """
        self.df = pd.read_csv(path)

    def data_update(self, Insert: bool) -> bool:
        """
        Please enter whether or not to update new data.
        """
        if Insert == True:
            self.df["opert_sctin_cd"] = "I"
        elif Insert == False:
            self.df["opert_sctin_cd"] = "U"

    def date_update(self):
        """
        Function to input date update of new data
        """
        self.df["data_crtin_dt"] = "".join(str(datetime.date.today()).split("-"))

    def CheckDate(self):
        """
        Investing.com's function to standardize data dates \n
        General use only
        """
        for TimeLength in range(len(self.df)):
            time = self.df["entrp_reltn_tdngs_dt"][TimeLength]
            test = time.split()
            if len(time.split()) != 5:
                TimeStamp = test[5] + self.MonthDict[test[3].replace("(", "")] + test[4]
            else:
                TimeStamp = test[2] + self.MonthDict[test[0]] + test[1]
            self.df["entrp_reltn_tdngs_dt"][TimeLength] = re.sub(r'[^0-9]','',TimeStamp)
    
    def CheckLength(self):
        '''
        If it is larger than the size of the data, a function that cuts it to the size of the data \n
        Both financial and general
        '''
        for key, value in self.definition.items():
            for Length in range(len(self.df["keyval"])):
                check = str(self.df[key.lower()][Length])
                
                if len(check) > value:
                    self.df[key.lower()][Length] = str(self.df[key.lower()][Length])[:value]
                else:
                    pass

    def CheckVarchar(self, News=False): 
        """
        A function that checks the data size and inserts the data if it is larger than the data size \n
        finance only
        """
        FirstRange = len(self.df['keyval'])
        cnt = 0
        
        for Length in range(FirstRange, FirstRange + cnt):
            self.df[self.ColumnsDict["상세"].lower()][Length] = self.df[self.ColumnsDict["상세"].lower()][Length][4000:]
            self.df[self.ColumnsDict["출처"].lower()][Length] = self.df[self.ColumnsDict["출처"].lower()][Length][4000:]
            self.df[self.ColumnsDict["사내"].lower()][Length] = self.df[self.ColumnsDict["사내"].lower()][Length][4000:]
            self.df[self.ColumnsDict["고객"].lower()][Length] = self.df[self.ColumnsDict["고객"].lower()][Length][4000:]  
            self.df[self.ColumnsDict["공급"].lower()][Length] = self.df[self.ColumnsDict["공급"].lower()][Length][4000:]
            self.df[self.ColumnsDict["경쟁"].lower()][Length] = self.df[self.ColumnsDict["경쟁"].lower()][Length][4000:]   
            self.df[self.ColumnsDict["대체"].lower()][Length] = self.df[self.ColumnsDict["대체"].lower()][Length][4000:]

        if News == True:
            for finan in self.Financial:
                self.df[finan.lower()] = np.nan
                self.df["entrp_reltn_tdngs_kind_cont"] = "Stock News"
            for Length in range(FirstRange):
                if len(str(self.df[self.ColumnsDict["날짜"].lower()][Length])) > 8:
                    date = str(self.df[self.ColumnsDict["날짜"].lower()][Length]).split()[:3]
                    if date[0] not in self.Month:
                        if len(date) < 8:
                            rlst = ["20221206", "20221205", "20221204"]
                            date = rlst[random.randrange(0, 2)]
                            self.df[self.ColumnsDict["날짜"].lower()][Length] = date
                        else:
                            date = str(self.df[self.ColumnsDict["날짜"].lower()][Length]).split()[3:6]
                            monthly = str(self.Month.index(date[0].replace("(", "")) + 1).rjust(2, "0")
                            date = date[2] + monthly + date[1].replace(",", "")
                            self.df[self.ColumnsDict["날짜"].lower()][Length] = date
                    else:
                        monthly = str(self.Month.index(date[0]) + 1).rjust(2, "0")
                        date = date[2] + monthly + date[1].replace(",", "")
                        self.df[self.ColumnsDict["날짜"].lower()][Length] = date
            if len(str(self.df[self.ColumnsDict["상세"].lower()][Length])) > 4000:
                try:
                    self.df[self.ColumnsDict["상세"].lower()][Length] = self.df[self.ColumnsDict["상세"].lower()][Length][:4000]
                    self.df[self.ColumnsDict["출처"].lower()][Length] = self.df[self.ColumnsDict["출처"].lower()][Length][:4000]
                    self.df[self.ColumnsDict["사내"].lower()][Length] = self.df[self.ColumnsDict["사내"].lower()][Length][:4000]
                    self.df[self.ColumnsDict["고객"].lower()][Length] = self.df[self.ColumnsDict["고객"].lower()][Length][:4000]
                    self.df[self.ColumnsDict["URL"].lower()][Length] = self.df[self.ColumnsDict["URL"].lower()][Length][:1000]  
                    self.df[self.ColumnsDict["공급"].lower()][Length] = self.df[self.ColumnsDict["공급"].lower()][Length][:4000]
                    self.df[self.ColumnsDict["경쟁"].lower()][Length] = self.df[self.ColumnsDict["경쟁"].lower()][Length][:4000]   
                    self.df[self.ColumnsDict["대체"].lower()][Length] = self.df[self.ColumnsDict["대체"].lower()][Length][:4000]
                    self.df = self.df.append(self.df.iloc[Length:Length+1], ignore_index = True)
                    cnt += 1
                except:
                    pass
            else: pass


    def CheckNumeric(self):
        """
        A function that determines whether it is a number, deletes it if it is not a number, and derives only the number if it contains a number \n
        finance only
        """
        numeric_match = re.compile('[^0-9]')
        for i in range(len(self.df)):
            for column in self.NumericList:
                result_lst = []
                try:
                    split_lst = self.df[column.lower()][i].split()
                    for idx in split_lst:
                        if numeric_match.match(idx) == None:
                            result_lst.append(re.sub(r'[^0-9]', '', idx))
                    if len(result_lst) >= 1:
                        if column == "STACNT_DT":
                            if 18000000 < int(result_lst[0]) < 21000000:
                                self.df[column.lower()][i] = result_lst[0]
                            else:
                                self.df[column.lower()][i] = ""    
                        else:
                            self.df[column.lower()][i] = result_lst[0]
                    else:
                        pass
                except:
                    pass

class Analysis:
    def __init__(self):
        """
        A module that analyzes data and saves error rates in Excel format
        """
        self.TableDefaultColumns = const.TableDefaultColumns
        self.TableDefaultColumns_info = const.TableDefaultColumns_info
        self.TableDefault = const.TableDefault
        self.DefaultTables = pd.DataFrame(self.TableDefault)
        self.DefaultColumns = pd.DataFrame(self.TableDefaultColumns)
        self.DefaultColumns_info = pd.DataFrame(self.TableDefaultColumns_info)
        self.ColumnDF = self.DefaultColumns.T
        self.ColumnDF = self.ColumnDF.rename(columns=self.ColumnDF.iloc[0])
        self.ColumnDF = self.ColumnDF.drop(self.ColumnDF.index[0])
        self.ColumnDF_info = self.DefaultColumns_info.T
        self.ColumnDF_info = self.ColumnDF_info.rename(columns=self.ColumnDF_info.iloc[0])
        self.ColumnDF_info = self.ColumnDF_info.drop(self.ColumnDF_info.index[0])
        self.ExceptList = ["stock_mrkt_cd", "acplc_lngg_stock_mrkt_nm", "engls_stock_mrkt_nm", "hb_ntn_cd", "crrnc_sctin_cd", 
                    "accnn_yr", "reprt_kind_cd","stacnt_dt", "opert_sctin_cd", "data_crtin_dt",
                    "cntct_prces_stts_cd", "cntct_prces_dt"]
        self.count = 0
        self.CheckDF = pd.DataFrame(columns=["번호", "시스템", "업무구분", "검증구분", "테이블명", "테이블ID", "컬럼명", "컬럼ID",
                                        "pk", "FK", "NN", "데이터타입", "도메인소분류",
                                        "도메인명", "전체건수", "오류건수", "오류율", "점검일시", "오류데이터"])
        self.DateList = list(self.DefaultColumns[self.DefaultColumns["데이터타입"] == "VARCHAR(8)"]["표준_영문컬럼명"].values)
        self.CheckDate = datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S')
    def find_keys_by_value(dictionary, value):
        keys = []
        for k, v in dictionary.items():
            if v == value:
                keys.append(k)
        return keys
    
    def read_excel(self, path):
        """
        Enter the excel address in path.
        """
        self.df = pd.read_excel(path)
        if "/" in path: Table = path.split("/")[-1].split(".")[0]
        else: Table = path.split(".")[0]
        self.TableName = list(self.DefaultTables[self.DefaultTables["영문 테이블명"] == Table]["한국 테이블명"])[0]
        self.TableID = str(Table)

    def read_csv(self, path):
        """
        Enter the csv address in path.
        """
        self.df = pd.read_csv(path)
        if "/" in path: Table = path.split("/")[-1].split(".")[0]
        else: Table = path.split(".")[0]
        self.TableName = list(self.DefaultTables[self.DefaultTables["영문 테이블명"] == Table]["한국 테이블명"])[0]
        self.TableID = str(Table)

    def Fail(self, column, Failed, info):
        """
        Function to input error data
        column = column in error
        Failed = erroneous data
        """ 
        if info == True:
            ColumnName = self.ColumnDF_info[column].values[0] # 컬럼명
            ColumnDataType = self.ColumnDF_info[column].values[1] # 데이터타입
            FailedDict = {
                            "번호" : self.count,
                            "시스템" : "공통",
                            "업무구분" : "헤브론스타",
                            "검증구분" : self.verification,
                            "테이블명" : self.TableName,
                            "테이블ID" : self.TableID,
                            "컬럼명" : ColumnName,
                            "컬럼ID" : column,
                            "pk" : "N",
                            "FK" : "N",
                            "NN" : "N",
                            "데이터타입" : ColumnDataType,
                            "도메인소분류" : "",
                            "도메인명" : "",
                            "전체건수" : len(self.df),
                            "오류건수" : len(self.df[self.df[column] == Failed]),
                            "오류율" : round(len(self.df[self.df[column] == Failed]) / len(self.df), 3),
                            "점검일시" : self.CheckDate,
                            "오류데이터" : Failed
                        }
            return FailedDict

        else:
            ColumnName = self.ColumnDF[column].values[0] # 컬럼명
            ColumnDataType = self.ColumnDF[column].values[1] # 데이터타입
            FailedDict = {
                            "번호" : self.count,
                            "시스템" : "공통",
                            "업무구분" : "헤브론스타",
                            "검증구분" : self.verification,
                            "테이블명" : self.TableName,
                            "테이블ID" : self.TableID,
                            "컬럼명" : ColumnName,
                            "컬럼ID" : column,
                            "pk" : "N",
                            "FK" : "N",
                            "NN" : "N",
                            "데이터타입" : ColumnDataType,
                            "도메인소분류" : "",
                            "도메인명" : "",
                            "전체건수" : len(self.df),
                            "오류건수" : len(self.df[self.df[column] == Failed]),
                            "오류율" : round(len(self.df[self.df[column] == Failed]) / len(self.df), 3),
                            "점검일시" : self.CheckDate,
                            "오류데이터" : Failed
                        }
            return FailedDict

    def CheckDate_Duplicate(self, info=False):
        """
        A function that checks for duplicates at the same time as the date.
        """
        if info==True:
            self.df.columns = list(self.DefaultColumns_info["표준_영문컬럼명"].values)        
            for column in tqdm(self.df.columns): # 컬럼ID
                for length in range(len(self.df[column])):
                    self.verification = "날짜"
                    if column in self.DateList:
                        try: pd.to_datetime(str(self.df[column][length]), format='%Y%m%d')
                        except:
                            self.count += 1
                            Failed = self.df[column][length]
                            Returned = self.Fail(column, Failed, info)
                            self.CheckDF = self.CheckDF.append(Returned, ignore_index=True)
                if column in self.ExceptList: continue
                else:
                    for cwt, idx in zip(self.df[column].value_counts(), self.df[column].value_counts().index):
                        per = cwt / len(self.df)
                        self.verification = "중복"
                        if per >= 0.05:
                            self.count += 1
                            Failed = idx
                            Returned = self.Fail(column, Failed, info)
                            self.CheckDF = self.CheckDF.append(Returned, ignore_index=True)

        else:
            self.df.columns = list(self.DefaultColumns["표준_영문컬럼명"].values)        
            for column in tqdm(self.df.columns): # 컬럼ID
                for length in range(len(self.df[column])):
                    self.verification = "날짜"
                    if column in self.DateList:
                        try: pd.to_datetime(str(self.df[column][length]), format='%Y%m%d')
                        except:
                            self.count += 1
                            Failed = self.df[column][length]
                            Returned = self.Fail(column, Failed)
                            self.CheckDF = self.CheckDF.append(Returned, ignore_index=True)
                if column in self.ExceptList: continue
                else:
                    for cwt, idx in zip(self.df[column].value_counts(), self.df[column].value_counts().index):
                        per = cwt / len(self.df)
                        self.verification = "중복"
                        if per >= 0.05:
                            self.count += 1
                            Failed = idx
                            Returned = self.Fail(column, Failed)
                            self.CheckDF = self.CheckDF.append(Returned, ignore_index=True)

    def CheckNumber(self, phone_columns, cop_columns):
        """
        Function to check columns of phone number
        """
        PN_1 = re.compile("\d{2,3}-\d{3,4}-\d{4}$") # phone number ex 1 000-0000-0000
        PN_2 = re.compile("\(\d{2,3}\)-\d{3,4}-\d{3,4}$") # phone number ex 2 (000)-0000-0000
        PN_3 = re.compile("\d{1,2} \d{7,8}$") # phone number ex 3 00 00000000
        BS_1 = re.compile("\d{1,2}-\d{7,8}$") # corporate number ex 1 00-00000000
        for column in phone_columns:
            for idx in range(len(self.df[column])):
                PN_Check_1 = PN_1.search(self.df[column][idx])
                PN_Check_2 = PN_2.search(self.df[column][idx])
                PN_Check_3 = PN_3.search(self.df[column][idx])
                if PN_Check_1 or PN_Check_2 or PN_Check_3:
                    pass
                else:
                    self.verification = "전화번호오류"
                    Failed = self.df[column][idx]
                    Returned = self.Fail(column, Failed)
                    self.CheckDF = self.CheckDF.append(Returned, ignore_index=True)
        for column in cop_columns:
            for idx in range(len(self.df[column])):
                BS_Check_1 = BS_1.search(self.df[column][idx])
                if BS_Check_1:
                    pass
                else:
                    self.verification = "법인번호오류"
                    Failed = self.df[column][idx]
                    Returned = self.Fail(column, Failed)
                    self.CheckDF = self.CheckDF.append(Returned, ignore_index=True)

    def change_columns(self, df, info=False, korea=False):
        """
        A function that converts collected data from standard Korean column names to English column names
        """
        if info==True: df.columns = [self.TableDefaultColumns_info[col] for col in df.columns]
        else: df.columns = [self.TableDefaultColumns["표준_영문컬럼명"][self.TableDefaultColumns["표준_한글컬렴명"].index(col)] if col in self.TableDefaultColumns["표준_한글컬렴명"] else col for col in df.columns]
        if korea==True: df.rename(columns={'법인등록번호': 'ovrss_entrp_crprt_rgno', '사업자등록번호': 'ovrss_entrp_bsnsm_rgno'}, inplace=True)
        return df


import pandas as pd

class information:
    def __init__(self):
        self.print_information()

    def print_information(self):
        print("""
        A description of the function is given below. \n
        The main class within the library is datos_extact. \n
        make() is a function that processes data. \n
        load() is a function that stores data.
        """)

class datos_extract:
    def __init__(self, path):
        """
        Please put TableDefaultColumns Excel in the path.
        """
        self.DefaultDataFrame = pd.DataFrame(columns = list(pd.read_excel(path, sheet_name="일반_columns")["표준_영문컬럼명"]))

    def make(self, path):
        """
        Function to standardize data
        Put the excel downloaded from Datos in the path.
        """
        df = pd.read_csv(path)
        AppendDict = {
            "hb_ntn_cd" : "COL",
            "acplc_lngg_ntn_nm" : "Colombia",
            "engls_ntn_nm" : "Colombia",
            "ntn_lngg_cd_val" : "COL",
            "acplc_lngg_lngg_nm" : "Colombia",
            "engls_lngg_nm" : "Colombia",
            "acplc_lngg_entrp_nm" : None,
            "engls_entrp_nm" : None,
            "acplc_lngg_oln_intrd_cont" : None,
            "acplc_lngg_entrp_intrd_cont" : None,
            "engls_oln_intrd_cont" : None,
            "engls_entrp_intrd_cont" : None,
            "rprsn_email" : None,
            "acplc_lngg_entrp_addr" : None,
            "acplc_lngg_entrp_dtadd" : None,
            "engls_entrp_addr" : None,
            "engls_entrp_dtadd" : None,
            "acplc_lngg_ceo_nm" : None, 
            "engls_ceo_nm" : None,
            "fndtn_dt" : None
        }
        company_email = list(df["EMAIL-COMERCIAL"])
        company_ceo = list(df["NOM-REP-LEGAL"])
        company_address = list(df["DIR-COMERCIAL"])
        company_name = list(df["RAZON SOCIAL"])
        AppendDict["acplc_lngg_entrp_nm"] = company_name
        AppendDict["engls_entrp_nm"] = company_name
        AppendDict["acplc_lngg_ceo_nm"] = company_ceo
        AppendDict["engls_ceo_nm"] = company_ceo
        AppendDict["acplc_lngg_entrp_addr"] = company_address
        AppendDict["acplc_lngg_entrp_dtadd"] = company_address
        AppendDict["engls_entrp_addr"] = company_address
        AppendDict["engls_entrp_dtadd"] = company_address
        AppendDict["rprsn_email"] = company_email
        AppendDataFrame = pd.DataFrame(AppendDict)
        self.DefaultDataFrame = self.DefaultDataFrame.append(AppendDataFrame, ignore_index=True)

    def load(self):
        """
        function to store data
        """
        self.DefaultDataFrame.to_excel("tb_hb_col_egnin_m.xlsx", index=False)


from sqlalchemy import create_engine
import pandas as pd

class information:
    def __init__(self):
        self.print_information()

    def print_information(self):
        print("""
        A description of the function is given below. \n
        The main class within the library is DataExtract. \n
        Enter database id, pw, port, database, table_name in order to connect.\n
        The connect() function is a function that tries to connect.\n
        The extract() function extracts the database after connecting.
        """)


class DataExtract:
    def __init__(self, id, pw, ip, pt, db, table_name):
        """
        postgresql ID, password, connection ip, database name \n
        please enter
        """
        self.id = id
        self.pw = pw
        self.ip = ip
        self.pt = pt
        self.db = db
        self.table_name = table_name

    def connect(self):
        """
        Function to connect to database
        """
        self.url = f"postgresql://{self.id}:{self.pw}@{self.ip}:{self.pt}/{self.db}"
        self.engine = create_engine(self.url)
    
    def extract(self):
        """
        Function to extract data from database
        """
        df = pd.read_sql_table(table_name = self.table_name, con=self.engine)
        return df.to_excel(f"{self.table_name}.xlsx")


import urllib, os

class information:
    def __init__(self):
        self.print_information()

    def print_information(self):
        print("""
        A description of the function is given below. \n
        The main class in the library is map. \n
        GetStreet() is a function that calls the Google map api. \n
        collect() is a function that extracts data.
        """)

class map:
    def __init__(self, api_key):
        self.key = api_key
        self.base = "https://maps.googleapis.com/maps/api/streetview"
        
    def GetStreet(self, address, save_loc):
        params = {"size":"1920x1080", "location":address, "key":self.key}
        url = self.base + "?" + urllib.parse.urlencode(params)
        if "\n" in address: address = address.replace("\n", "")
        filename = address.replace(",", "").replace(" ", "_") + ".bmp"
        urllib.request.urlretrieve(url, os.path.join(save_loc, filename))

    def collect(self, address_lst, save_loc):
        for address in address_lst:
            self.GetStreet(address, save_loc)


import requests
from .constants import const
import pandas as pd
import json
try:
    # For Python 3.0 and later
    from urllib.request import urlopen
except ImportError:
    # Fall back to Python 2's urllib2
    from urllib2 import urlopen

import certifi
from tqdm import tqdm
from datetime import datetime
import math
import warnings
warnings.simplefilter(action='ignore', category=FutureWarning)

class information:
    def __init__(self):
        self.print_information()

    def print_information(self):
        print("""
        A description of the function is given below. \n
        The main class within the library is fmp_extact. \n
        get_jsonparsed_data() is a function that parses data. \n
        extractor() is a function that brings data in json format. \n
        url_generator() is a function that connects to the FMP site and separates data. \n
        ending_period_extact() is a function that normalizes dates. \n
        report_type_extract() is a function that determines whether it is annual or quarterly based on the incoming value. \n
        GetExcel() is a function that stores the extracted data. \n
        cleanse() is a function that processes data. \n
        get_symbols() is a function that gets data from the site. \n
        make_clean() is a function that sequentially executes the above functions to extract and save data.
        """)

class fmp_extract:
    def __init__(self):
        self.core_cols = const.fmp_core_cols 
        self.is_cols = const.fmp_is_cols 
        self.bs_cols = const.fmp_bs_cols
        self.cf_cols = const.fmp_cf_cols
        self.FMP_field = const.fmp_FMP_field


    def get_jsonparsed_data(self, url):
        """
        Receive the content of ``url``, parse it as JSON and return the object.

        Parameters
        ----------
        url : str

        Returns
        -------
        dict
        """
        response = urlopen(url, cafile=certifi.where())
        data = response.read().decode("utf-8")
        return json.loads(data)

    def extractor(self, url) : 
        try:
            from pandas import json_normalize
        except ImportError:
            from pandas.io.json import json_normalize
        req=requests.get(url)
        # json load
        data = json.loads(req.text)
        # json to pandas
        preprocessed = json_normalize(data)
        return preprocessed

    def url_generator(self, target_Symbol, filing_type, limit, api_key, period='quarter') : 
        base_url = f'https://financialmodelingprep.com/api/v3/income-statement/RY.TO?limit=5&period=quarter&apikey={api_key}'
        if period == 'annual' : 
            base_url = base_url.replace('&period=quarter','')
        
        is_url = base_url.replace('RY.TO', target_Symbol)
        is_url = is_url.replace('=5', '='+str(limit))
        bs_url = is_url.replace('income-statement','balance-sheet-statement')
        cf_url = is_url.replace('income-statement','cash-flow-statement')
        
        if filing_type == 'is' : 
            return is_url
        elif filing_type == 'bs' : 
            return bs_url
        elif filing_type == 'cf' : 
            return cf_url 
        else :
            return "Please input proper filing_type"

    def ending_period_extract(self, x) : 
        try : 
            date = datetime.strptime(str(x), '%Y-%m-%d')
            return date.strftime("%Y%m%d")
        except : 
            return 

    def report_type_extract(self, x) : 
        try : 
            if 'Q' in str(x) : 
                return 'Q'
            elif 'FY' in str(x) : 
                return 'A'
            else : 
                return 
        except : 
            return 

    def cleanse(self, path, filename, fund = False, trading = True) : 
        original_df = pd.read_excel(path)
        """
        data processing function
        original_df : Original data extracted from extract_and_save \n
        save_dir : Directory to save \n
        fund: True if you want to check ETFs listed on the stock market, otherwise False \n
        trading: True if you want to extract only companies that are currently trading in the stock market, otherwise False
        """

        if fund == True : 
            pass  # included fund
        else : 
            # false if you want to see only real listed company information with funds removed
            original_df['isFund'].fillna(False) 
            original_df = original_df[original_df['isFund']== False]
            
            
        # Extract only the companies that are currently trading
        if trading == True : 
            # Assume that firms with isActivelyTrading blank are trading
            original_df['isActivelyTrading'] = original_df['isActivelyTrading'].fillna(True)
            original_df = original_df[original_df['isActivelyTrading']==True]
        else : 
            pass
        
        
        ## Get correspondence table
        mapping_sheet = pd.DataFrame(self.FMP_field)
        # Dictionize correspondence table key : value = standard field name : FMP field name
        mapping_dic = mapping_sheet.set_index('채워야할 테이블 필드명').T.to_dict('index')['Financial Modeling API']
        
        processed = pd.DataFrame(columns = mapping_dic.keys())
        
        # Use the correspondence table with the loop below to insert information into an empty data frame called processed.
        for order, field in enumerate(processed.columns) : 
            try : 
                processed.loc[:,field] = original_df.loc[:,mapping_dic[field]]
            except : 
                pass
        import re 
        regExp = '\W[a-zA-Z]+'  # Remove special characters that are not letters and numbers with \W and '' with [a-zA-Z]+. Remove stock market code behind
        processed['lstng_cd']= processed['lstng_cd'].str.replace(pat = regExp ,repl=r'', regex = True)
        processed['stacnt_dt'] = processed['stacnt_dt'].apply(lambda x : self.ending_period_extract(x))
        processed['lstng_dt'] = processed['lstng_dt'].apply(lambda x : self.ending_period_extract(x))
        processed['fndtn_dt'] = processed['fndtn_dt'].apply(lambda x : self.ending_period_extract(x))
        processed['reprt_kind_cd'] = processed['reprt_kind_cd'].apply(lambda x : self.report_type_extract(x))
        
        processed.to_excel(filename, index = False)
        print(f"{filename}이 완료되었습니다.")
        
        return processed

    def get_symbols(self, country, api_key):
        """
        데이터 가져오는 함수 \n
        county에 국가명을 입력하세요. \n
        api_key is fmp api key
        """
        # Query String to get Symbols of companies with Financial Statements.
        url = (f"https://financialmodelingprep.com/api/v3/financial-statement-symbol-lists?apikey={api_key}")
        Symbols = self.get_jsonparsed_data(url)

        US_symbols = []          # 
        Canada_symbols = []      #TO
        France_symbols = []      #PA
        Germany_symbols = []     #DE
        India_symbols = []       #NS
        London_symbols = []      #L
        Hongkong_symbols = []    #HK
        Australia_symbols = []  #AX
        Swiss_symbols = []       #SW
        Korea_symbols = []       #KR
        Netherlands_symbols = []

        for Symbol in Symbols : 
            if ".TO" in Symbol : 
                Canada_symbols.append(Symbol) 
            elif ".PA" in Symbol : 
                France_symbols.append(Symbol) 
            elif ".DE" in Symbol : 
                Germany_symbols.append(Symbol) 
            elif ".NS" in Symbol : 
                India_symbols.append(Symbol) 
            elif ".BS" in Symbol : 
                India_symbols.append(Symbol) 
            elif ".L" in Symbol : 
                London_symbols.append(Symbol) 
            elif ".HK" in Symbol : 
                Hongkong_symbols.append(Symbol) 
            elif ".AX" in Symbol : 
                Australia_symbols.append(Symbol) 
            elif ".SW" in Symbol : 
                Swiss_symbols.append(Symbol) 
            elif ".KS" in Symbol :
                Korea_symbols.append(Symbol)
            elif "." not in Symbol :
                US_symbols.append(Symbol) 
            elif ".EU" in Symbol : 
                Netherlands_symbols.append(Symbol)


        cnt = 0
        if country == "캐나다" : selected_symbols = Canada_symbols
        elif country == "프랑스" : selected_symbols = France_symbols
        elif country == "독일" : selected_symbols = Germany_symbols
        elif country == "인도" : selected_symbols = India_symbols
        elif country == "영국" : selected_symbols = London_symbols
        elif country == "홍콩" : selected_symbols = Hongkong_symbols
        elif country == "호주" : selected_symbols = Australia_symbols
        elif country == "스위스" : selected_symbols = Swiss_symbols
        elif country == "한국" : selected_symbols = Korea_symbols
        elif country == "미국" : selected_symbols = US_symbols
        elif country == "네덜란드" : selected_symbols = Netherlands_symbols
        selected_symbols = [i for i in selected_symbols if "\n" not in i]

        for _ in range(math.ceil(len(selected_symbols)/1000)):
            company_df_list = pd.DataFrame()
            for target_Symbol in tqdm(selected_symbols[cnt:cnt+1000]) :  
                if cnt == len(selected_symbols):
                    company_df_list = company_df_list.sort_values(by=['symbol','date'], ascending = [True,False]) # Sort symbols in ascending order, fiscal quarters within the same symbol in descending order.       
                    # As of 2022, only 5 years of data, that is, data from 2017 or more are extracted. -> calendarYear (fiscal year) is more than 2017
                    # company_df_list = company_df_list[company_df_list['calendarYear'].astype('int')>=2022]
                    company_df_list.to_excel(f'{country}_{cnt}.xlsx', index=False)

                for report_type in ['annual','quarter'] :
                
                    if report_type == 'annual' : 
                        limit = 2 # Annual financial statement 1 year
                    else : 
                        limit = 8 # Quarterly financial statements (4 quarters per year)
                
            
                    is_url = self.url_generator(target_Symbol, 'is', limit, api_key, report_type)
                    bs_url = self.url_generator(target_Symbol, 'bs', limit, api_key, report_type)
                    cf_url = self.url_generator(target_Symbol, 'cf', limit, api_key, report_type)

                    # Use the extractor function to import data through the API into a Pandas data frame.
                    df_is = self.extractor(is_url)
                    df_bs = self.extractor(bs_url)
                    df_cf = self.extractor(cf_url)


                # Error Avoidance
                # Occasionally, there are problems such as income statement and balance sheet, but no cash flow statement.
                # Error avoidance for all cases
                # try: If there is information about the columns to be extracted, call it
                # except: if there is information about the columns to be extracted, core_is()

                    try :  
                        df_is = df_is[self.core_cols+self.is_cols]
                    except : 
                        df_is = pd.DataFrame(columns=self.core_cols+self.is_cols)    
                    try :     
                        df_bs= df_bs[self.core_cols+self.bs_cols]
                    except : 
                        df_bs = pd.DataFrame(columns=self.core_cols+self.bs_cols)

                    try : 
                        df_cf = df_cf[self.core_cols+self.cf_cols]
                    except : 
                        df_cf = pd.DataFrame(columns=self.core_cols+self.cf_cols)


                    # merge above three dataframes 
                    #company_df = pd.concat([df_is,df_bs,df_cf],axis = 1)
                    company_df = pd.merge(df_is, df_bs, how = 'outer', on = self.core_cols)
                    company_df = pd.merge(company_df, df_cf, how = 'outer', on = self.core_cols)

                    # Add necessary company general information in addition to numeric data.
                    # Use Company profile API of Financil Modeling Prep.
                    base_url = f'https://financialmodelingprep.com/api/v3/profile/RY.TO?apikey={api_key}'
                    target_url = base_url.replace('RY.TO', target_Symbol)

                    # Get data with request
                    req = requests.get(target_url)
                    data = json.loads(req.text)

                    # Extract only necessary general information columns based on the data specification and add them to company_df.

                    try : company_df['companyName'] = data[0]['companyName'] 
                    except :company_df['companyName'] = ""
                    try : company_df['ceo'] = data[0]['ceo']
                    except :company_df['ceo'] = ""
                    try : company_df['phone'] =data[0]['phone']
                    except : company_df['phone'] = ""
                    try : company_df['website'] =data[0]['website']
                    except : company_df['website'] = ""
                    try : company_df['state'] =data[0]['state']
                    except : company_df['state'] = ""
                    try : company_df['city'] =data[0]['city']
                    except : company_df['city'] = ""
                    try : company_df['country'] =data[0]['country']
                    except : company_df['country'] = ""
                    try : company_df['industry'] =data[0]['industry']
                    except : company_df['industry'] = ""
                    try : company_df['ipoDate'] =data[0]['ipoDate']
                    except : company_df['ipoDate'] = ""
                    try : company_df['address'] =data[0]['address']
                    except : company_df['address'] = ""
                    try : company_df['zip'] =data[0]['zip']
                    except : company_df['zip'] = ""
                    try : company_df['exchangeShortName'] =data[0]['exchangeShortName']
                    except : company_df['exchangeShortName'] = ""
                    try : company_df['exchange'] =data[0]['exchange']
                    except :  company_df['exchange'] = ""
                    try : company_df['description'] =data[0]['description']
                    except : company_df['description'] = ""
                    try : company_df['isEtf'] =data[0]['isEtf']
                    except : company_df['isEtf'] = ""
                    try : company_df['isActivelyTrading'] =data[0]['isActivelyTrading']
                    except : company_df['isActivelyTrading'] = ""
                    try : company_df['isFund'] =data[0]['isFund']
                    except : company_df['isFund'] = ""
                        
                    # Add tax number. Use 'Company core information API'.
                    try : 
                        base_url = f'https://financialmodelingprep.com/api/v4/company-core-information?symbol=AAPL&apikey={api_key}'
                        target_url = base_url.replace('AAPL', target_Symbol)

                        # Get data with request
                        # If the imported data is null, an error will occur in the process of extracting information from 'data'.
                        req = requests.get(target_url)
                        data = json.loads(req.text)
                        
                        company_df['taxIdentificationNumber'] =data[0]['taxIdentificationNumber']
                        company_df['registrantName'] =data[0]['registrantName']

                    except : 
                        company_df['taxIdentificationNumber'] = ""
                        company_df['registrantName'] = ""
                    
                        
                        
                    company_df_list = company_df_list.append(company_df)
                    cnt += 1
                
                
            company_df_list = company_df_list.sort_values(by=['symbol','date'], ascending = [True,False]) # Sort symbols in ascending order, fiscal quarters within the same symbol in descending order.  

            # As of 2022, only 5 years of data, that is, data from 2017 or more are extracted. -> calendarYear (fiscal year) is more than 2017
            # company_df_list = company_df_list[company_df_list['calendarYear'].astype('int')>=2022]
            company_df_list.to_excel(f'Original_{country}_{cnt}.xlsx', index=False)

    def make_clean(self, FilePath, SavePath):
        """
        FilePath = directory to be cleaned \n
        SavePath = Directory to save after Clean operation
        """
        for File in tqdm(FilePath):
            name = File.split(".")[0]
            self.cleanse(f"{SavePath}/{File}", name)


import pandas as pd
import os
import psycopg2
from .constants import const
from sqlalchemy import create_engine
from tqdm import tqdm

class information:
    def __init__(self):
        self.print_information()

    def print_information(self):
        print("""
        The DataLoad() class is the main one. \n
        The class can handle large amounts if many = True is set. \n
        DataLoading() is a function that saves data in the form of a data frame within a class. \n
        CheckLength() is a function that measures the length of the saved data frame to prevent errors beyond the standard. In addition, the value of keyval is raised above the latest value that currently exists. \n
        Load() loads the data using a batch process. \n
        Login() is a function that connects to the database. \n
        Connect_DB() is a function that connects to the database and creates an environment where data can be loaded.
        """)

class DataLoad:
    """
    A module that loads data into the database
    Set many to True when there is a lot of data to use.
    """
    def __init__(self, many=False):
        """
        When loading multiple files, set many = True.
        """
        self.country = const.country
        self.dtypesql_finan = const.dtypesql_finan
        self.definition_finan = const.definition_finan
        self.dtypesql_info = const.dtypesql_info
        self.definition_info = const.definition_info
        self.empty_data = const.stock_name
        self.many = many
        self.url = None
        self.df = None
        self.table_name = None
        self.replace = False
        self.first = False
        self.table_nameList = []
        self.DataFrameList = []

    def DataLoading(self, Path):
        """
        A function used by setting a path when loading two or more data
        """
        FilePath = os.listdir(Path)
        if self.many == True:
            for file in tqdm(FilePath):
                tmp_table_name = file.split(".")[0]
                self.table_nameList.append(tmp_table_name)
                filetype = file.split(".")[1]
                if filetype == "csv":
                    try:
                        tmp = pd.read_csv(Path+file, encoding = "cp949")
                    except:
                        tmp = pd.read_csv(Path+file)
                elif filetype == "xlsx":
                    tmp = pd.read_excel(Path+file)       
                self.DataFrameList.append(tmp)
        else:
            for file in tqdm(FilePath):
                self.table_name = file.split(".")[0]
                filetype = file.split(".")[1]
                if filetype == "csv":
                    try:
                        self.df = pd.read_csv(Path+file, encoding = "cp949")
                    except:
                        self.df = pd.read_csv(Path+file)
                elif filetype == "xlsx":
                    self.df = pd.read_excel(Path+file)       

    def CheckLength(self):
        '''
        If it is larger than the data size, a function that cuts the data to that size
        Korea Finance No
        '''
        if self.many == False:
            if self.table_name.split("_")[-1] == "m":
                definition = self.definition_info
            elif self.table_name.split("_")[-1] == "d":
                definition = self.definition_finan 
            for key, value in tqdm(definition.items()):
                for Length in range(len(self.df)):
                    check = str(self.df[key.lower()][Length])
                    if check == "nan":
                        continue
                    elif len(check) > value:
                        self.df[key.lower()][Length] = str(self.df[key.lower()][Length])[:value]
                    else:
                        pass
        else:
            for length in tqdm(range(len(self.table_nameList))):
                if self.table_nameList[length].split("_")[-1] == "m":
                    definition = self.definition_info
                elif self.table_nameList[length].split("_")[-1] == "d":
                    definition = self.definition_finan 
                for key, value in tqdm(definition.items()):
                    for Length in range(len(self.DataFrameList[length])):
                        check = str(self.DataFrameList[length][key.lower()][Length])
                        if check == "nan":
                            continue
                        elif len(check) > value:
                            self.DataFrameList[length][key.lower()][Length] = str(self.DataFrameList[length][key.lower()][Length])[:value]
                        else:
                            pass


    def Load(self):
        """
        Use a batch process. \n
        This is a process, also known as batch processing, where \n is processed by request in real time.
        It processes large amounts of data in batches rather than in a way.
        """
        engine = create_engine(self.url)
        if self.many == False:
            if self.table_name.split("_")[-1] == "m":
                dtypesql = self.dtypesql_info
            elif self.table_name.split("_")[-1] == "d":
                dtypesql = self.dtypesql_finan
            if self.replace == False:
                self.df.to_sql(name = self.table_name, con=engine, schema='public',chunksize= 10000,
                if_exists='append', index = False, dtype=dtypesql, method = 'multi')
            elif self.replace == True:
                self.df.to_sql(name = self.table_name, con=engine, schema='public',chunksize= 10000,
                if_exists='replace', index = False, dtype=dtypesql, method = 'multi')
            return f"{self.table_name} has finished loading."
        else:
            for length in tqdm(range(len(self.table_nameList))):
                if self.table_nameList[length].split("_")[-1] == "m":
                    dtypesql = self.dtypesql_info
                elif self.table_nameList[length].split("_")[-1] == "d":
                    dtypesql = self.dtypesql_finan
                if self.replace == False:
                    self.DataFrameList[length].to_sql(name = self.table_nameList[length], con=engine, schema='public',chunksize= 10000,
                    if_exists='append', index = False, dtype=dtypesql, method = 'multi')
                elif self.replace == True:
                    self.DataFrameList[length].to_sql(name = self.table_nameList[length], con=engine, schema='public',chunksize= 10000,
                    if_exists='replace', index = False, dtype=dtypesql, method = 'multi')
                print(f"{self.table_nameList[length]} has finished loading.")


    def Login(self, user, password, host, port, dbname):
        self.url = f"postgresql://{user}:{password}@{host}:{port}/{dbname}"

    def Connect_DB(self, replace=False, first=False):
        """
        required to connect to postgresql \n
        id, password, host, port, dbname.\n
        replace means whether to update. \n
        You can choose to proceed with the update. \n
        It is necessary because the table name is different for each country.
        """
        self.replace = replace
        self.first = first
        if self.many == False:
            if first == False:
                try:
                    conn = psycopg2.connect(self.url)
                    cur = conn.cursor()
                    cur.execute(f"select keyval from {self.table_name} order by keyval desc limit 1;")
                    rows = cur.fetchall()
                    NowKeyval = int(str(rows[0]).split("'")[1])
                except psycopg2.DatabaseError as db_err:
                    print(f"The current error is {db_err}")
                for keyval, idx in zip(range(NowKeyval, len(self.df)+NowKeyval), range(len(self.df))):
                    self.df["keyval"][idx] = int(keyval)
                self.df["keyval"] = self.df["keyval"].astype(int)
            else:
                for Length in tqdm(range(len(self.df))):
                    self.df["keyval"][Length] = int(Length)
                self.df["keyval"] = self.df["keyval"].astype(int)
        else:
            for length in tqdm(range(len(self.table_nameList))):
                if first == False:
                    try:
                        conn = psycopg2.connect(self.url)
                        cur = conn.cursor()
                        cur.execute(f"select keyval from {self.table_nameList[length]} order by keyval desc limit 1;")
                        rows = cur.fetchall()
                        NowKeyval = int(str(rows[0]).split("'")[1]) + 1
                    except psycopg2.DatabaseError as db_err:
                        print(f"The current error is {db_err}")
                    for keyval, idx in zip(range(NowKeyval, len(self.DataFrameList[length])+NowKeyval), range(len(self.DataFrameList[length]))):
                        self.DataFrameList[length]["keyval"][idx] = int(keyval)
                    self.DataFrameList[length]["keyval"] = self.DataFrameList[length]["keyval"].astype(int)
                else:
                    for Length in tqdm(range(len(self.DataFrameList[length]))):
                        self.DataFrameList[length]["keyval"][Length] = int(Length)
                    self.DataFrameList[length]["keyval"] = self.DataFrameList[length]["keyval"].astype(int)
        
    def fill_data(self):
        for key, value in self.empty_data.items():
            for df_name, df in zip(self.table_nameList, self.DataFrameList):
                if key not in df_name:
                    continue
                df.update(df[['stock_mrkt_cd', 'acplc_lngg_stock_mrkt_nm', 'engls_stock_mrkt_nm']].fillna(value))
                df['hb_ntn_cd'] = df['hb_ntn_cd'].fillna(df_name.split('_')[2].upper())

    def change_name(self, df_name):
        df_name = next((f"tb_hb_{value}_plcfi_d.xlsx" for key, value in self.country.items() if key in df_name), df_name)
        return df_name


import yfinance as yf
import psycopg2
import pandas as pd

class information:
    def __init__(self):
        self.print_information()

    def print_information(self):
        print("""
        The yfinance library collects market cap data. \n
        collect() is a function that collects data from shareoutstanding sites. \n
        """)

class YFINANCE:
    def __init__(self, country, host, database, user, password):
        conn = psycopg2.connect(
                host = host,
                database = database,
                user = user,
                password = password
                )
        self.query = f"SELECT * FROM tb_hb_{country}_plcfi_d"
        self.df = pd.read_sql(self.query, self.conn)
        self.company_lst = [i for i in list(self.df["lstng_cd"].unique()) if i is not None]
    
    def collect(self, path):
        for name in self.company_lst:
            ticker = yf.Ticker(name)
            hist = ticker.history(period="1d", start="2018-01-01")
            hist = hist.reset_index()
            hist["Date"] = hist["Date"].astype(str)
            hist.to_excel(f"{path}/"+ name + "_5yr.xlsx", index= False)


import io
from setuptools import find_packages, setup
from os import path


# --- get version ---
version = "unknown"
with open("dinnovation/version.py") as f:
    line = f.read().strip()
    version = line.replace("version = ", "").replace("'", "")
    
# --- /get version ---

# --- get requirements ---
requirements = None
with open("requirements.txt") as f:
    requirements = f.read().splitlines()

here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with io.open(path.join(here, 'README.md'), encoding="utf-8") as f:
    long_description = f.read()

setup(
    name="dinnovation",
    version=version,
    author="cmblir",
    author_email="sodlalwl13@gmail.com",
    long_description = long_description,
    long_description_content_type = "text/markdown",
    description="Digital Industry Innovation Data Platform Big data collection and processing, database loading, distribution",
    classifiers=[
        "Programming Language :: Python :: 3.8",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
        'Intended Audience :: Developers',
        'Topic :: Office/Business :: Financial',
        'Topic :: Office/Business :: Financial :: Investment',
        'Topic :: Scientific/Engineering :: Interface Engine/Protocol Translator',
        'Topic :: Software Development :: Libraries',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
    include_package_data=True,
    packages=find_packages(),
    platforms=['any'],
    python_requires='>=3.9',
    install_requires=requirements
)

print("""
NOTE: dinnovation is not affiliated, endorsed, or vetted by source sites.""")


import pandas as pd
from .constants import const
from tqdm import tqdm
import OpenDartReader

class information:
    def __init__(self):
        self.print_information()

    def print_information(self):
        print("""
        A description of the function is given below. \n
        The main class within the library is dart_extract. \n
        api_key() is a function that informs the api key. \n
        extract_finstate() is a function that extracts data. \n
        load_finstate() is a function that saves data.
        """)

class dart_extract:
    def __init__(self, Path):
        """
        Download all items from data at http://data.krx.co.kr/contents/MDC/MDI/mdiLoader. \n
        Enter the downloaded data file path.
        """
        self.Columns = const.korea_financial_column
        self.cleansed_finstats = pd.DataFrame(columns= self.Columns)
        if Path.split(".")[-1] == "xlsx": self.df = pd.read_excel(Path)
        elif Path.split(".")[-1] == "csv": self.df = pd.read_csv(Path, encoding='cp949')
        self.listed_companies = self.df[self.df['주식종류']=='보통주']["한글 종목약명"]
        self.failed_companies = []
        
    def extract_finstate(self, name, reprt, api_key):
        """
        For name, enter the company name through the received data. \n
        Enter the values ["11013", "11012", "11014", "11011"] in reprt. \n
        Quarterly, Semiannually, Thirdly, Annually, in order.\n
        Select and insert one of the four into the api key through api_key().
        """
        dart = OpenDartReader(api_key)
        success_num = 0
        failed_num = 0
        self.success_columns = []
        self.add_index = []
        try:
            self.founded_finstats = pd.DataFrame(dart.finstate_all(name, 2022, reprt_code = reprt).set_index("account_nm").T)
        except:
            self.failed_companies.append(name)
            return f"{name}기업의 재무정보를 찾을 수 없습니다."
        self.company_info1 = dart.company(name)
        # cleansed_finstats['키값'] = np.nan
        self.success_columns.append('주식시장코드')
        self.success_columns.append('현지언어주식시장명')
        self.success_columns.append('영문주식시장명')
        self.success_columns.append('헤브론스타국가코드')
        self.success_columns.append('상장코드')
        self.success_columns.append('현지언어기업명')
        self.success_columns.append('영문기업명')
        self.success_columns.append('법인등록번호')
        self.success_columns.append('사업자등록번호')
        self.success_columns.append('설립일자')
        self.success_columns.append('현지언어산업군명')
        self.success_columns.append('통화구분코드')
        self.success_columns.append('회계연도')
        self.success_columns.append('보고서종류코드')
        self.success_columns.append('결산일자')
        market_segmentation = str(self.df[self.df["한글 종목약명"] == name]["시장구분"]).split()[1]
        self.add_index.append(market_segmentation)
        if market_segmentation == 'KOSDAQ':
            self.add_index.append("코스닥시장")
        elif market_segmentation == 'KOSPI':
            self.add_index.append("유가증권시장")
        elif market_segmentation == 'KONEX':
            self.add_index.append("코넥스시장")
        try:
            self.add_index.append(market_segmentation)
            self.add_index.append("KOR")
            self.add_index.append(self.company_info1['stock_code'])
            self.add_index.append(self.company_info1["corp_name"])
            self.add_index.append(self.company_info1['corp_name_eng'])
            self.add_index.append(self.company_info1['jurir_no'])
            self.add_index.append(self.company_info1['bizr_no'])
            self.add_index.append(self.company_info1['est_dt'])
            self.add_index.append(self.company_info1['induty_code'])
            self.add_index.append("KRW")
            self.add_index.append("2022")
        except:
            return
        try:
            if '분기' in self.founded_finstats["유동자산"]:
                self.add_index.append("Q")
            else:
                self.add_index.append("A")
        except:
            pass
        try:
            self.add_index.append(self.founded_finstats['유동자산'].loc['rcept_no'][:8])
        except:
            self.add_index.append("19991118")
            
        for idx in self.Columns:
            if idx == "없음":
                pass
            try:
                self.add_index.append(self.founded_finstats[idx].loc["thstrm_amount"])
                self.success_columns.append(idx)
                success_num += 1
            except KeyError:
                failed_num += 1
        while len(self.success_columns) > len(self.add_index):
            if len(self.success_columns) == len(self.add_index): break
            else: self.add_index.append("0")
        self.complete_df = pd.DataFrame([tuple(self.add_index)], columns=self.success_columns)
        self.cleansed_finstats = self.cleansed_finstats.append(self.complete_df, ignore_index=True)

    def load_finstats(self, api_key):
        """
        Function to store data \n
        Select and insert one of the four into the api key through api_key().
        """
        for name in tqdm(self.listed_companies):
            for num in ["11013", "11012", "11014"]:
                self.extract_finstate(name, num, api_key)
        self.cleansed_finstats.to_excel("KOREA.xlsx", index=False)


import chromedriver_autoinstaller
import warnings
import pandas as pd
import time
from selenium import webdriver
from webdriver_manager.chrome import ChromeDriverManager
from selenium.webdriver.chrome.service import Service
from selenium.webdriver.common.by import By
from selenium.webdriver.support.select import Select
warnings.simplefilter(action='ignore', category=FutureWarning)
warnings.filterwarnings("ignore", category=DeprecationWarning) 

class information:
    def __init__(self):
        self.print_information()

    def print_information(self):
        print("""
        A description of the function is given below. \n
        The main class within the library is opencorporates_extract. \n
        DriverSettings() is a function that sets the driver. \n
        Login() is a function to log in to opencorporates. \n
        ReCounty() is a function that selects a country. \n
        SearchCompanies() is a function that finds companies. \n
        GetInformation() is a function that extracts data. \n
        GetExcel() is a function that stores the extracted data. \n
        """)

class country_name:
    def __init__(self):
        self.print_information()

    def print_information(self):
        print("""
        Hong Kong, Malaysia, Singapore, Thailand \n
        Viet Nam, India, United States, Canada \n
        Mexico, Netherlands, Germany, Italy, France \n
        United Kingdom, Australia, Switzerland, Spain
        """)


class opencorporates_extract:
    def __init__(self):
        self.url = "https://opencorporates.com/"
        self.accounturl = "https://opencorporates.com/users/account"
        self.CompaniesInformationUrl = pd.DataFrame(columns=["country", "company name", "url"])
        self.CompaniesInformation = pd.DataFrame()
    
    def DriverSettings(self, Turn_off_warning = False, linux_mode = False) -> None:
        """
        This function sets the driver.
        If linux mode is set to True, collection is possible in the background.
        However, actions such as clicks cannot be taken.
        """
        if Turn_off_warning == True: self.TurnOffWarning()
        chrome_ver = chromedriver_autoinstaller.get_chrome_version().split('.')[0]  # Check chromedriver version
        chrome_options = webdriver.ChromeOptions()
        chrome_options.add_argument("--incognito") # incognito mode
        if linux_mode == True: chrome_options.add_argument("--headless") # Display mode without Linux GUI
        chrome_options.add_argument("--no-sandbox") # Prevent access to resources
        chrome_options.add_argument("--disable-setuid-sandbox") # Prevent chrome crashes
        chrome_options.add_argument("--disable-dev-shm-usage") # Prevent out of memory errors
        chrome_options.add_experimental_option('excludeSwitches', ['enable-logging'])
        try: # Chrome Driver
            self.driver = webdriver.Chrome(service=Service(ChromeDriverManager().install()), options=chrome_options)   
        except:
            chromedriver_autoinstaller.install(True)
            self.driver = webdriver.Chrome(service=Service(ChromeDriverManager().install()), options=chrome_options)
        # Prevent WebDruverException Error Designate as an existing driver version
    
    def Login(self, id, pw):
        """
        Function to log in opencorporates
        Enter id and pw.
        """
        self.driver.get(self.accounturl)
        time.sleep(3)
        EmailAddress = self.driver.find_element(By.XPATH, '//*[@id="user_email"]')
        EmailPassword = self.driver.find_element(By.XPATH, '//*[@id="user_password"]')
        EmailAddress.send_keys(id)
        EmailPassword.send_keys(pw)
        SIGN = self.driver.find_element(By.XPATH, '//*[@id="new_user"]/div[5]/div/button')
        SIGN.send_keys("\n")
    
    def ReCountry(self, CountryName):
        """
        Function to select country
        Please enter the country name in CountryName.
        """
        self.driver.get(self.url)
        Countries = Select(self.driver.find_element(By.NAME, "jurisdiction_code"))
        Countries.select_by_visible_text(CountryName)
        CountriesApply = self.driver.find_element(By.XPATH, '//*[@id="home-background-circle"]/div/div[2]/div/div[1]/form/div[2]/div[2]/div[3]/button')
        CountriesApply.send_keys("\n")
        ExcludeInactive = self.driver.find_element(By.XPATH, '//*[@id="inactive"]')
        ExcludeInactive.send_keys(" ")
        GoSearch = self.driver.find_element(By.XPATH, '//*[@id="basic_companies_search"]/div[2]/input[2]')
        GoSearch.send_keys("\n")
    
    def SearchCompanies(self, CountryName):
        """
        A function to search for a company and extract the url
        Please enter the country name in CountryName.
        """
        TotalCompanies = int(self.driver.find_element(By.XPATH, '//*[@id="page_container"]/div[2]/div[1]/h2').text.split()[1].replace(",", ""))
        for _ in range(TotalCompanies // 30):
            results = self.driver.find_element(By.ID, "results")
            results = results.find_element(By.TAG_NAME, "ul")
            results = results.find_elements(By.TAG_NAME, "li")
            for result in results:
                Aclass = result.find_elements(By.TAG_NAME, "a")
                CompanyName = Aclass[1].text
                CompanyUrl = Aclass[1].get_attribute("href")
                df = pd.DataFrame({"country" : CountryName, "company name" : CompanyName, "url" : CompanyUrl}, index=[0])
                self.CompaniesInformationUrl = self.CompaniesInformationUrl.append(df, ignore_index=True)
            NextPage = self.driver.find_element(By.XPATH, '//*[@id="results"]/div/div[1]/ul/li[8]/a').get_attribute("href")
            self.driver.get(NextPage)
        self.CompaniesInformationUrl.to_excel(f"finished_{CountryName}_url_opencorporates.xlsx", index = False, engine='xlsxwriter')
    
    def GetInformation(self, url, CountryName):
        """
        A function that searches for companies in url and extracts information
        Please enter the country name in CountryName.
        """
        self.driver.get(url)
        vcard = self.driver.find_element(By.CLASS_NAME, "vcard")
        CompanyName = vcard.find_element(By.TAG_NAME, "h1").text
        TableName = vcard.find_elements(By.TAG_NAME, "dt")
        TableIndex = vcard.find_elements(By.TAG_NAME, "dd")
        TableDict = {"Country" : CountryName, "Company Name" : CompanyName}
        for table, index in zip(TableName, TableIndex):
            TableDict[table.text] = index.text
        Tabledf = pd.DataFrame(TableDict, index=[0])
        self.CompaniesInformation = self.CompaniesInformation.append(Tabledf, ignore_index = True)

    def GetExcel(self):
        """
        function to store data
        """
        self.CompaniesInformation.to_excel("finished_opencorporates.xlsx", index=False, engine="xlsxwriter")